@@ -82,6 +82,9 @@ GPU::MosaicLookup GPU::mosaicLookup;
 //#define DEBUG_TRI
 
 CACHE_ALIGN u8 GPU_screen[4*256*192];
+CACHE_ALIGN u8 GPU_tempScreen[4*256*192];
+CACHE_ALIGN u8 GPU_tempScanline[4*256]; //only one scanline
+bool GPU_tempScanline_valid; //whether GPU_tempScanline is valid (if not, fetch it from GPU_screen)
 
 CACHE_ALIGN u8 sprWin[256];
 
@@ -89,6 +92,8 @@ OSDCLASS	*osd = NULL;
 OSDCLASS	*osdA = NULL;
 OSDCLASS	*osdB = NULL;
 
+u16			gpu_angle = 0;
+
 const short sizeTab[4][4][2] =
 {
       {{256,256}, {512, 256}, {256, 512}, {512, 512}},
@@ -317,6 +322,10 @@ void GPU_Reset(GPU *g, u8 l)
 	osd = new OSDCLASS(-1);
 	//DISP_FIFOclear(&g->disp_fifo);
 	memset(GPU_screen, 0, sizeof(GPU_screen));
+	memset(GPU_tempScreen, 0, sizeof(GPU_tempScreen));
+
+	for(int i = 0; i < (256*192*2); i++)
+		((u16*)GPU_screen)[i] = 0x7FFF;
 }
 
 void GPU_DeInit(GPU * gpu)
@@ -446,16 +455,16 @@ void GPU_setVideoProp(GPU * gpu, u32 p)
 	switch (gpu->dispMode)
 	{
 		case 0: // Display Off
-			return;
+			break;
 		case 1: // Display BG and OBJ layers
 			break;
 		case 2: // Display framebuffer
 			gpu->VRAMaddr = (u8 *)ARM9Mem.ARM9_LCD + (gpu->vramBlock * 0x20000);
-			return;
+			break;
 		case 3: // Display from Main RAM
 			// nothing to be done here
 			// see GPU_ligne who gets data from FIFO.
-			return;
+			break;
 	}
 
 	if(cnt->OBJ_Tile_mapping)
@@ -2784,19 +2793,19 @@ void GPU_set_DISPCAPCNT(u32 val)
 	switch((val >> 20) & 0x03)
 	{
 		case 0:
-			gpu->dispCapCnt.capx = 128;
+			gpu->dispCapCnt.capx = DISPCAPCNT::_128;
 			gpu->dispCapCnt.capy = 128;
 			break;
 		case 1:
-			gpu->dispCapCnt.capx = 256;
+			gpu->dispCapCnt.capx = DISPCAPCNT::_256;
 			gpu->dispCapCnt.capy = 64;
 			break;
 		case 2:
-			gpu->dispCapCnt.capx = 256;
+			gpu->dispCapCnt.capx = DISPCAPCNT::_256;
 			gpu->dispCapCnt.capy = 128;
 			break;
 		case 3:
-			gpu->dispCapCnt.capx = 256;
+			gpu->dispCapCnt.capx = DISPCAPCNT::_256;
 			gpu->dispCapCnt.capy = 192;
 			break;
 	}
@@ -2814,7 +2823,6 @@ static void GPU_ligne_layer(NDS_Screen * screen, u16 l)
 {
 	GPU * gpu = screen->gpu;
 	struct _DISPCNT * dispCnt = &(gpu->dispx_st)->dispx_DISPCNT.bits;
-	gpu->currDst = (u8 *)(GPU_screen) + (screen->offset + l) * 512;
 	itemsForPriority_t * item;
 	u8 spr[512];
 	u8 sprAlpha[256];
@@ -2966,6 +2974,20 @@ static void GPU_ligne_layer(NDS_Screen * screen, u16 l)
 // TODO: capture emulated not fully
 static void GPU_ligne_DispCapture(u16 l)
 {
+	//this macro takes advantage of the fact that there are only two possible values for capx
+	#define CAPCOPY(SRC,DST) \
+	switch(gpu->dispCapCnt.capx) { \
+		case DISPCAPCNT::_128: \
+			for (int i = 0; i < 128; i++)  \
+				T2WriteWord(DST, i << 1, T2ReadWord(SRC, i << 1) | (1<<15)); \
+			break; \
+		case DISPCAPCNT::_256: \
+			for (int i = 0; i < 256; i++)  \
+				T2WriteWord(DST, i << 1, T2ReadWord(SRC, i << 1) | (1<<15)); \
+			break; \
+			default: assert(false); \
+		}
+	
 	GPU * gpu = MainScreen.gpu;
 
 	if (l == 0)
@@ -2996,27 +3018,20 @@ static void GPU_ligne_DispCapture(u16 l)
 								{
 									//INFO("Capture screen (BG + OBJ + 3D)\n");
 
-									//here we have a special hack. if the main screen is in an unusual display mode
-									//(other than the regular layer combination mode)
-									//then we havent combined layers including the 3d.
-									//in that case, we need to skip straight to the capture 3d only
-									if(MainScreen.gpu->dispMode != 1)
-										goto cap3d;
-
-									u8 *src = (u8 *)(GPU_screen) + (MainScreen.offset + l) * 512;
-									for (int i = 0; i < gpu->dispCapCnt.capx; i++)
-										T2WriteWord(cap_dst, i << 1, T2ReadWord(src, i << 1) | (1<<15));
-
+									u8 *src;
+									if(GPU_tempScanline_valid)
+										src = (u8*)(GPU_tempScanline);
+									else 
+										src = (u8 *)(GPU_tempScreen + (MainScreen.offset + l) * 512);
+									CAPCOPY(src,cap_dst);
 								}
 							break;
 							case 1:			// Capture 3D
 								{
 									//INFO("Capture 3D\n");
-								cap3d:
 									u16 cap3DLine[512];
 									gpu3D->NDS_3D_GetLineCaptured(l, (u16*)cap3DLine);
-									for (int i = 0; i < gpu->dispCapCnt.capx; i++)
-										T2WriteWord(cap_dst, i << 1, (u16)cap3DLine[i]);
+									CAPCOPY(((u8*)cap3DLine),cap_dst);
 								}
 							break;
 						}
@@ -3031,8 +3046,7 @@ static void GPU_ligne_DispCapture(u16 l)
 								{
 									//INFO("Capture VRAM\n");
 									u8 *src = (u8 *)(gpu->dispCapCnt.src) + (l * 512);
-									for (int i = 0; i < gpu->dispCapCnt.capx; i++)
-										T2WriteWord(cap_dst, i << 1, T2ReadWord(src, i << 1) | (1<<15));
+									CAPCOPY(src,cap_dst);
 								}
 								break;
 							case 1:			// Capture Main Memory Display FIFO
@@ -3050,8 +3064,14 @@ static void GPU_ligne_DispCapture(u16 l)
 						u16 *srcB = NULL;
 						u16 cap3DLine[512];
 
-						if (gpu->dispCapCnt.srcA == 0)			// Capture screen (BG + OBJ + 3D)
-							srcA = (u16 *)(GPU_screen) + (MainScreen.offset + l) * 512;
+						if (gpu->dispCapCnt.srcA == 0)
+						{
+							// Capture screen (BG + OBJ + 3D)
+							if(GPU_tempScanline_valid)
+								srcA = (u16*)(GPU_tempScanline);
+							else 
+								srcA = (u16*)((u8 *)GPU_tempScreen + (MainScreen.offset + l) * 512);
+						}
 						else
 						{
 							gpu3D->NDS_3D_GetLineCaptured(l, (u16*)cap3DLine);
@@ -3066,7 +3086,8 @@ static void GPU_ligne_DispCapture(u16 l)
 						if ((srcA) && (srcB))
 						{
 							u16 a, r, g, b;
-							for(u16 i = 0; i < gpu->dispCapCnt.capx; i++) 
+							const int todo = (gpu->dispCapCnt.capx==DISPCAPCNT::_128?128:256);
+							for(u16 i = 0; i < todo; i++) 
 							{
 								a = r = g = b =0;
 
@@ -3112,7 +3133,7 @@ static INLINE void GPU_ligne_MasterBrightness(NDS_Screen * screen, u16 l)
 {
 	GPU * gpu = screen->gpu;
 
-	u8 * dst =  GPU_screen + (screen->offset + l) * 512;
+	u8 * dst =  GPU_tempScreen + (screen->offset + l) * 512;
 	u16 i16;
 
 	if (!gpu->MasterBrightFactor) return;
@@ -3248,6 +3269,7 @@ void GPU::update_winh(int WIN_NUM)
 void GPU_ligne(NDS_Screen * screen, u16 l)
 {
 	GPU * gpu = screen->gpu;
+	GPU_tempScanline_valid = false;
 
 	//here is some setup which is only done on line 0
 	if(l == 0) {
@@ -3296,7 +3318,7 @@ void GPU_ligne(NDS_Screen * screen, u16 l)
 	{
 		case 0: // Display Off(Display white)
 			{
-				u8 * dst =  GPU_screen + (screen->offset + l) * 512;
+				u8 * dst =  GPU_tempScreen + (screen->offset + l) * 512;
 
 				for (int i=0; i<256; i++)
 					T2WriteWord(dst, i << 1, 0x7FFF);
@@ -3304,19 +3326,20 @@ void GPU_ligne(NDS_Screen * screen, u16 l)
 			break;
 
 		case 1: // Display BG and OBJ layers
+				screen->gpu->currDst = (u8 *)(GPU_tempScreen) + (screen->offset + l) * 512;
 				GPU_ligne_layer(screen, l);
 			break;
 
 		case 2: // Display framebuffer
 			{
-				u8 * dst = GPU_screen + (screen->offset + l) * 512;
+				u8 * dst = GPU_tempScreen + (screen->offset + l) * 512;
 				u8 * src = gpu->VRAMaddr + (l*512);
 				memcpy (dst, src, 512);
 			}
 			break;
 		case 3: // Display memory FIFO
 			{
-				u8 * dst =  GPU_screen + (screen->offset + l) * 512;
+				u8 * dst =  GPU_tempScreen + (screen->offset + l) * 512;
 				for (int i=0; i < 128; i++)
 					T1WriteLong(dst, i << 2, DISP_FIFOrecv() & 0x7FFF7FFF);
 			}
@@ -3325,6 +3348,17 @@ void GPU_ligne(NDS_Screen * screen, u16 l)
 
 	if (gpu->core == GPU_MAIN) 
 	{
+		//when not displaying the BG and OBJ, we might still need to generate it
+		//for purposes of capturing. therefore:
+		//TODO - LOW PRIORITY OPTIMIZATION - based on capture settings, selectively generate this
+		//we have already optimized it a tiny bit by putting it under the if(core is main)
+		if(gpu->dispMode != 1)
+		{
+			screen->gpu->currDst = (u8 *)(GPU_tempScanline);
+			GPU_ligne_layer(screen, l);
+			GPU_tempScanline_valid = true;
+		}
+
 		GPU_ligne_DispCapture(l);
 		if (l == 191) { disp_fifo.head = disp_fifo.tail = 0; }
 	}
@@ -3333,12 +3367,12 @@ void GPU_ligne(NDS_Screen * screen, u16 l)
 
 void gpu_savestate(std::ostream* os)
 {
-	os->write((char*)GPU_screen,sizeof(GPU_screen));
+	os->write((char*)GPU_tempScreen,sizeof(GPU_tempScreen));
 }
 
 bool gpu_loadstate(std::istream* is)
 {
-	is->read((char*)GPU_screen,sizeof(GPU_screen));
+	is->read((char*)GPU_tempScreen,sizeof(GPU_tempScreen));
 	MainScreen.gpu->updateBLDALPHA();
 	SubScreen.gpu->updateBLDALPHA();
 	return !is->fail();
@@ -3350,6 +3384,54 @@ void GPU::setAffineStart(int layer, int xy, u32 val)
 	else affineInfo[layer-2].y = val;
 }
 
+void gpu_UpdateRender()
+{
+	int x = 0, y = 0;
+	u16 *src = (u16*)GPU_tempScreen;
+	u16	*dst = (u16*)GPU_screen;
+
+	switch (gpu_angle)
+	{
+		case 0:
+			memcpy(dst, src, 256*192*4);
+			break;
+
+		case 90:
+			for(y = 0; y < 384; y++)
+			{
+				for(x = 0; x < 256; x++)
+				{
+					dst[(383 - y) + (x * 384)] = src[x + (y * 256)];
+				}
+			}
+			break;
+		case 180:
+			for(y = 0; y < 384; y++)
+			{
+				for(x = 0; x < 256; x++)
+				{
+					dst[(255 - x) + ((383 - y) * 256)] = src[x + (y * 256)];
+				}
+			}
+			break;
+		case 270:
+			for(y = 0; y < 384; y++)
+			{
+				for(x = 0; x < 256; x++)
+				{
+					dst[y + ((255 - x) * 384)] = src[x + (y * 256)];
+				}
+			}
+		default:
+			break;
+	}
+}
+
+void gpu_SetRotateScreen(u16 angle)
+{
+	gpu_angle = angle;
+}
+
 //here is an old bg mosaic with some old code commented out. I am going to leave it here for a while to look at it
 //sometimes in case I find a problem with the mosaic.
 //static void __setFinalColorBck(GPU *gpu, u32 passing, u8 bgnum, u8 *dst, u16 color, u16 x, bool opaque)
