@@ -342,15 +342,17 @@ typedef union
     source:
     http://nocash.emubase.de/gbatek.htm#dsvideocaptureandmainmemorydisplaymode
 *******************************************************************************/
-typedef struct
+struct DISPCAPCNT
 {
+	enum CAPX {
+		_128, _256
+	} capx;
     u32		val;
 	BOOL	enabled;
 	u8		EVA;
 	u8		EVB;
 	u8		writeBlock;
 	u8		writeOffset;
-	u16		capx;
 	u16		capy;
 	u8		srcA;
 	u8		srcB;
@@ -359,7 +361,7 @@ typedef struct
 	u8		capSrc;
 	u8		*dst;
 	u8		*src;
-} DISPCAPCNT;
+} ;
 
 /*******************************************************************************
     this structure holds everything and should be mapped to
@@ -427,9 +429,9 @@ typedef union
 
 struct _ROTOCOORD
 {
-/* 0*/	unsigned Fraction:8;
-/* 8*/	  signed Integer:24;
-// /*28*/	unsigned :4;
+	u32 Fraction:8;
+	s32 Integer:20;
+	u32 pad:4;
 };
 typedef union
 {
@@ -795,6 +797,7 @@ static void REG_DISPx_pack_test(GPU * gpu)
 */
 
 CACHE_ALIGN extern u8 GPU_screen[4*256*192];
+CACHE_ALIGN extern u8 GPU_tempScreen[4*256*192];
 
 
 GPU * GPU_Init(u8 l);
@@ -911,5 +914,9 @@ void SetupFinalPixelBlitter (GPU *gpu);
 #define GPU_setBGxHOFS(bg, gpu, val) gpu->dispx_st->dispx_BGxOFS[bg].BGxHOFS = (val & 0x1F)
 #define GPU_setBGxVOFS(bg, gpu, val) gpu->dispx_st->dispx_BGxOFS[bg].BGxVOFS = (val & 0x1F)
 
+// render
+void gpu_UpdateRender();
+void gpu_SetRotateScreen(u16 angle);
+
 #endif
 
