@@ -24,6 +24,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <algorithm>
+#include <math.h>
 
 #include "common.h"
 #include "NDSSystem.h"
@@ -45,6 +46,11 @@
 
 //#define USE_REAL_BIOS
 
+TCommonSettings CommonSettings;
+static Driver _stub_driver;
+Driver* driver = &_stub_driver;
+std::string InputDisplayString;
+
 static BOOL LidClosed = FALSE;
 static u8	countLid = 0;
 char pathToROM[MAX_PATH];
@@ -54,8 +60,324 @@ char ROMserial[20];
 /* the count of bytes copied from the firmware into memory */
 #define NDS_FW_USER_SETTINGS_MEM_BYTE_COUNT 0x70
 
+BOOL fw_success = FALSE;
+
 NDSSystem nds;
 
+int lagframecounter;
+int LagFrameFlag;
+int lastLag;
+int TotalLagFrames;
+
+/* ------------------------------------------------------------------------- */
+/* FIRMWARE DECRYPTION */
+
+static u32 keyBuf[0x412];
+static u32 keyCode[3];
+
+#define DWNUM(i) ((i) >> 2)
+
+static u32 bswap32(u32 val)
+{
+	return (((val & 0x000000FF) << 24) | ((val & 0x0000FF00) << 8) | ((val & 0x00FF0000) >> 8) | ((val & 0xFF000000) >> 24));
+}
+
+static BOOL getKeyBuf()
+{
+	int dummy;
+	FILE *file = fopen(CommonSettings.ARM7BIOS, "rb");
+
+	if(file == NULL)
+		return FALSE;
+
+	fseek(file, 0x30, SEEK_SET);
+	dummy = fread(keyBuf, 0x412, 4, file);
+
+	fclose(file);
+	return TRUE;
+}
+
+static void crypt64BitUp(u32 *ptr)
+{
+	u32 Y = ptr[0];
+	u32 X = ptr[1];
+
+	for(u32 i = 0x00; i <= 0x0F; i++)
+	{
+		u32 Z = (keyBuf[i] ^ X);
+		X = keyBuf[DWNUM(0x048 + (((Z >> 24) & 0xFF) << 2))];
+		X = (keyBuf[DWNUM(0x448 + (((Z >> 16) & 0xFF) << 2))] + X);
+		X = (keyBuf[DWNUM(0x848 + (((Z >> 8) & 0xFF) << 2))] ^ X);
+		X = (keyBuf[DWNUM(0xC48 + ((Z & 0xFF) << 2))] + X);
+		X = (Y ^ X);
+		Y = Z;
+	}
+
+	ptr[0] = (X ^ keyBuf[DWNUM(0x40)]);
+	ptr[1] = (Y ^ keyBuf[DWNUM(0x44)]);
+}
+
+static void crypt64BitDown(u32 *ptr)
+{
+	u32 Y = ptr[0];
+	u32 X = ptr[1];
+
+	for(u32 i = 0x11; i >= 0x02; i--)
+	{
+		u32 Z = (keyBuf[i] ^ X);
+		X = keyBuf[DWNUM(0x048 + (((Z >> 24) & 0xFF) << 2))];
+		X = (keyBuf[DWNUM(0x448 + (((Z >> 16) & 0xFF) << 2))] + X);
+		X = (keyBuf[DWNUM(0x848 + (((Z >> 8) & 0xFF) << 2))] ^ X);
+		X = (keyBuf[DWNUM(0xC48 + ((Z & 0xFF) << 2))] + X);
+		X = (Y ^ X);
+		Y = Z;
+	}
+
+	ptr[0] = (X ^ keyBuf[DWNUM(0x04)]);
+	ptr[1] = (Y ^ keyBuf[DWNUM(0x00)]);
+}
+
+static void applyKeycode(u32 modulo)
+{
+	crypt64BitUp(&keyCode[1]);
+	crypt64BitUp(&keyCode[0]);
+
+	u32 scratch[2] = {0x00000000, 0x00000000};
+
+	for(u32 i = 0; i <= 0x44; i += 4)
+	{
+		keyBuf[DWNUM(i)] = (keyBuf[DWNUM(i)] ^ bswap32(keyCode[DWNUM(i % modulo)]));
+	}
+
+	for(u32 i = 0; i <= 0x1040; i += 8)
+	{
+		crypt64BitUp(scratch);
+		keyBuf[DWNUM(i)] = scratch[1];
+		keyBuf[DWNUM(i+4)] = scratch[0];
+	}
+}
+
+static BOOL initKeycode(u32 idCode, int level, u32 modulo)
+{
+	if(getKeyBuf() == FALSE)
+		return FALSE;
+
+	keyCode[0] = idCode;
+	keyCode[1] = (idCode >> 1);
+	keyCode[2] = (idCode << 1);
+
+	if(level >= 1) applyKeycode(modulo);
+	if(level >= 2) applyKeycode(modulo);
+
+	keyCode[1] <<= 1;
+	keyCode[2] >>= 1;
+
+	if(level >= 3) applyKeycode(modulo);
+
+	return TRUE;
+}
+
+static u16 getBootCodeCRC16()
+{
+	int i, j;
+	u32 crc = 0xFFFF;
+	const u16 val[8] = {0xC0C1, 0xC181, 0xC301, 0xC601, 0xCC01, 0xD801, 0xF001, 0xA001};
+
+	for(i = 0; i < nds.FW_ARM9BootCodeSize; i++)
+	{
+		crc = (crc ^ nds.FW_ARM9BootCode[i]);
+
+		for(j = 0; j < 8; j++) 
+		{
+			if(crc & 0x0001)
+				crc = ((crc >> 1) ^ (val[j] << (7-j)));
+			else
+				crc =  (crc >> 1);
+		}
+	}
+
+	for(i = 0; i < nds.FW_ARM7BootCodeSize; i++)
+	{
+		crc = (crc ^ nds.FW_ARM7BootCode[i]);
+
+		for(j = 0; j < 8; j++) 
+		{
+			if(crc & 0x0001)
+				crc = ((crc >> 1) ^ (val[j] << (7-j)));
+			else
+				crc =  (crc >> 1);
+		}
+	}
+
+	return (crc & 0xFFFF);
+}
+
+static u32 * decryptFirmwareBlock(const char *blockName, u32 *src, u32 &size)
+{
+	u32 curBlock[2];
+	u32 *dst;
+	u32 i, j;
+	u32 xIn = 4, xOut = 0;
+	u32 len;
+	u32 offset;
+	u32 windowOffset;
+	u32 xLen;
+	u8 d;
+	u16 data;
+
+	memcpy(curBlock, src, 8);
+	crypt64BitDown(curBlock);
+
+	u32 blockSize = (curBlock[0] >> 8);
+	size = blockSize;
+
+	INFO("Firmware: %s final size: %i bytes\n", blockName, blockSize);
+
+	dst = (u32*)new u8[blockSize];
+
+	xLen = blockSize;
+
+	while(xLen > 0)
+	{
+		d = T1ReadByte((u8*)curBlock, (xIn % 8));
+		xIn++;
+		if((xIn % 8) == 0)
+		{
+			memcpy(curBlock, (((u8*)src) + xIn), 8);
+			crypt64BitDown(curBlock);
+		}
+
+		for(i = 0; i < 8; i++)
+		{
+			if(d & 0x80)
+			{
+				data = (T1ReadByte((u8*)curBlock, (xIn % 8)) << 8);
+				xIn++;
+				if((xIn % 8) == 0)
+				{
+					memcpy(curBlock, (((u8*)src) + xIn), 8);
+					crypt64BitDown(curBlock);
+				}
+				data |= T1ReadByte((u8*)curBlock, (xIn % 8));
+				xIn++;
+				if((xIn % 8) == 0)
+				{
+					memcpy(curBlock, (((u8*)src) + xIn), 8);
+					crypt64BitDown(curBlock);
+				}
+
+				len = (data >> 12) + 3;
+				offset = (data & 0xFFF);
+				windowOffset = (xOut - offset - 1);
+
+				for(j = 0; j < len; j++)
+				{
+					T1WriteByte((u8*)dst, xOut, T1ReadByte((u8*)dst, windowOffset));
+					xOut++;
+					windowOffset++;
+
+					xLen--;
+					if(xLen == 0)
+						goto lz77End;
+				}
+			}
+			else
+			{
+				T1WriteByte((u8*)dst, xOut, T1ReadByte((u8*)curBlock, (xIn % 8)));
+				xOut++;
+				xIn++;
+				if((xIn % 8) == 0)
+				{
+					memcpy(curBlock, (((u8*)src) + xIn), 8);
+					crypt64BitDown(curBlock);
+				}
+
+				xLen--;
+				if(xLen == 0)
+					goto lz77End;
+			}
+
+			d = ((d << 1) & 0xFF);
+		}
+	}
+
+lz77End:
+
+	return dst;
+}
+
+static BOOL decryptFirmware(u8 *data)
+{
+	u16 shifts;
+	u16 shift1, shift2, shift3, shift4;
+	u32 part1addr, part2addr, part3addr, part4addr, part5addr;
+	u32 part1ram, part2ram;
+
+	shifts = T1ReadWord(data, 0x14);
+	shift1 = (shifts & 0x7);
+	shift2 = ((shifts >> 3) & 0x7);
+	shift3 = ((shifts >> 6) & 0x7);
+	shift4 = ((shifts >> 9) & 0x7);
+
+	part1addr = T1ReadWord(data, 0x0C);
+	part1addr = (part1addr << (2+shift1));
+
+	INFO("Firmware: ARM9 boot code address: %05X\n", part1addr);
+
+	part1ram = T1ReadWord(data, 0x0E);
+	part1ram = (0x02800000 - (part1ram << (2+shift2)));
+
+	INFO("Firmware: ARM9 boot code RAM address: %08X\n", part1ram);
+
+	part2addr = T1ReadWord(data, 0x10);
+	part2addr = (part2addr << (2+shift3));
+
+	INFO("Firmware: ARM7 boot code address: %05X\n", part2addr);
+
+	part2ram = T1ReadWord(data, 0x12);
+	part2ram = (0x03810000 - (part2ram << (2+shift4)));
+
+	INFO("Firmware: ARM7 boot code RAM address: %08X\n", part2ram);
+
+	part3addr = T1ReadWord(data, 0x00);
+	part3addr = (part3addr << 3);
+
+	INFO("Firmware: ARM9 GUI code address: %05X\n", part3addr);
+
+	part4addr = T1ReadWord(data, 0x02);
+	part4addr = (part4addr << 3);
+
+	INFO("Firmware: ARM7 GUI code address: %05X\n", part4addr);
+
+	part5addr = T1ReadWord(data, 0x16);
+	part5addr = (part5addr << 3);
+
+	INFO("Firmware: data/gfx address: %05X\n", part5addr);
+
+	if(initKeycode(T1ReadLong(data, 0x08), 1, 0xC) == FALSE) return FALSE;
+	crypt64BitDown((u32*)&data[0x18]);
+	if(initKeycode(T1ReadLong(data, 0x08), 2, 0xC) == FALSE) return FALSE;
+
+	nds.FW_ARM9BootCode = (u8*)decryptFirmwareBlock("ARM9 boot code", (u32*)&data[part1addr], nds.FW_ARM9BootCodeSize);
+	nds.FW_ARM7BootCode = (u8*)decryptFirmwareBlock("ARM7 boot code", (u32*)&data[part2addr], nds.FW_ARM7BootCodeSize);
+
+	nds.FW_ARM9BootCodeAddr = part1ram;
+	nds.FW_ARM7BootCodeAddr = part2ram;
+
+	u16 crc16_header = T1ReadWord(data, 0x06);
+	u16 crc16_mine = getBootCodeCRC16();
+	if(crc16_header != crc16_mine)
+	{
+		INFO("Firmware: error: the boot code CRC16 (%04X) doesn't match the value in the firmware header (%04X).\n", 
+			crc16_mine, crc16_header);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+/* ------------------------------------------------------------------------- */
+
 static u32
 calc_CRC16( u32 start, const u8 *data, int count) {
 	int i,j;
@@ -175,6 +497,8 @@ int NDS_Init( void) {
 	nds.VCount = 0;
 	nds.lignerendu = FALSE;
 
+	nds.sleeping = FALSE;
+
 	if (Screen_Init(GFXCORE_DUMMY) != 0)
 		return -1;
 
@@ -193,8 +517,13 @@ int NDS_Init( void) {
 
 #ifdef EXPERIMENTAL_WIFI
 	WIFI_Init(&wifiMac) ;
+	if(wifiMac.netEnabled)
+		WIFI_SoftAP_Init(&wifiMac);
 #endif
 
+	nds.FW_ARM9BootCode = NULL;
+	nds.FW_ARM7BootCode = NULL;
+
 	return 0;
 }
 
@@ -207,6 +536,11 @@ void NDS_DeInit(void) {
 	Screen_DeInit();
 	MMU_DeInit();
 	gpu3D->NDS_3D_Close();
+
+#ifdef EXPERIMENTAL_WIFI
+	if(wifiMac.netEnabled)
+		WIFI_SoftAP_Shutdown(&wifiMac);
+#endif
 }
 
 BOOL NDS_SetROM(u8 * rom, u32 mask)
@@ -529,30 +863,119 @@ void NDS_Reset( void)
 
 	if (!header) return ;
 
+	lagframecounter=0;
+	LagFrameFlag=0;
+	lastLag=0;
+	TotalLagFrames=0;
+
 	MMU_clearMem();
 
-	src = header->ARM9src;
-	dst = header->ARM9cpy;
+	//ARM7 BIOS IRQ HANDLER
+	if(CommonSettings.UseExtBIOS == true)
+		inf = fopen(CommonSettings.ARM7BIOS,"rb");
+	else
+		inf = NULL;
 
-	for(i = 0; i < (header->ARM9binSize>>2); ++i)
-	{
-		_MMU_write32<ARMCPU_ARM9>(dst, T1ReadLong(MMU.CART_ROM, src));
-		dst += 4;
-		src += 4;
+	if(inf) {
+		fread(MMU.ARM7_BIOS,1,16384,inf);
+		fclose(inf);
+		if(CommonSettings.SWIFromBIOS == true) NDS_ARM7.swi_tab = 0;
+		else NDS_ARM7.swi_tab = ARM7_swi_tab;
+		INFO("ARM7 BIOS is loaded.\n");
+	} else {
+		NDS_ARM7.swi_tab = ARM7_swi_tab;
+		_MMU_write32<ARMCPU_ARM7>(0x00, 0xE25EF002);
+		_MMU_write32<ARMCPU_ARM7>(0x04, 0xEAFFFFFE);
+		_MMU_write32<ARMCPU_ARM7>(0x18, 0xEA000000);
+		_MMU_write32<ARMCPU_ARM7>(0x20, 0xE92D500F);
+		_MMU_write32<ARMCPU_ARM7>(0x24, 0xE3A00301);
+		_MMU_write32<ARMCPU_ARM7>(0x28, 0xE28FE000);
+		_MMU_write32<ARMCPU_ARM7>(0x2C, 0xE510F004);
+		_MMU_write32<ARMCPU_ARM7>(0x30, 0xE8BD500F);
+		_MMU_write32<ARMCPU_ARM7>(0x34, 0xE25EF004);
 	}
 
-	src = header->ARM7src;
-	dst = header->ARM7cpy;
+	//ARM9 BIOS IRQ HANDLER
+	if(CommonSettings.UseExtBIOS == true)
+		inf = fopen(CommonSettings.ARM9BIOS,"rb");
+	else
+		inf = NULL;
+	//memcpy(ARM9Mem.ARM9_BIOS + 0x20, gba_header_data_0x04, 156);
 
-	for(i = 0; i < (header->ARM7binSize>>2); ++i)
-	{
-		_MMU_write32<ARMCPU_ARM7>(dst, T1ReadLong(MMU.CART_ROM, src));
-		dst += 4;
-		src += 4;
+	if(inf) {
+		fread(ARM9Mem.ARM9_BIOS,1,4096,inf);
+		fclose(inf);
+		if(CommonSettings.SWIFromBIOS == true) NDS_ARM9.swi_tab = 0;
+		else NDS_ARM9.swi_tab = ARM9_swi_tab;
+		INFO("ARM9 BIOS is loaded.\n");
+	} else {
+		NDS_ARM9.swi_tab = ARM9_swi_tab;
+		_MMU_write32<ARMCPU_ARM9>(0xFFFF0018, 0xEA000000);
+		_MMU_write32<ARMCPU_ARM9>(0xFFFF0020, 0xE92D500F);
+		_MMU_write32<ARMCPU_ARM9>(0xFFFF0024, 0xEE190F11);
+		_MMU_write32<ARMCPU_ARM9>(0xFFFF0028, 0xE1A00620);
+		_MMU_write32<ARMCPU_ARM9>(0xFFFF002C, 0xE1A00600);
+		_MMU_write32<ARMCPU_ARM9>(0xFFFF0030, 0xE2800C40);
+		_MMU_write32<ARMCPU_ARM9>(0xFFFF0034, 0xE28FE000);
+		_MMU_write32<ARMCPU_ARM9>(0xFFFF0038, 0xE510F004);
+		_MMU_write32<ARMCPU_ARM9>(0xFFFF003C, 0xE8BD500F);
+		_MMU_write32<ARMCPU_ARM9>(0xFFFF0040, 0xE25EF004);
 	}
 
-	armcpu_init(&NDS_ARM7, header->ARM7exe);
-	armcpu_init(&NDS_ARM9, header->ARM9exe);
+	/* Is it really needed ??? */
+	_MMU_write32<ARMCPU_ARM9>(0x0000004, 0xE3A0010E);
+	_MMU_write32<ARMCPU_ARM9>(0x0000008, 0xE3A01020);
+	// _MMU_write32<ARMCPU_ARM9>(0x000000C, 0xE1B02110);
+	_MMU_write32<ARMCPU_ARM9>(0x000000C, 0xE1B02040);
+	_MMU_write32<ARMCPU_ARM9>(0x0000010, 0xE3B02020);
+	// _MMU_write32<ARMCPU_ARM9>(0x0000010, 0xE2100202);
+
+	if(CommonSettings.UseExtFirmware == true)
+		NDS_LoadFirmware(CommonSettings.Firmware);
+
+	if((CommonSettings.UseExtBIOS == true) && (CommonSettings.UseExtFirmware == true) && (CommonSettings.BootFromFirmware == true) && (fw_success == TRUE))
+	{
+		for(i = 0; i < nds.FW_ARM9BootCodeSize; i += 4)
+		{
+			_MMU_write32<ARMCPU_ARM9>((nds.FW_ARM9BootCodeAddr + i), T1ReadLong(nds.FW_ARM9BootCode, i));
+		}
+
+		for(i = 0; i < nds.FW_ARM7BootCodeSize; i += 4)
+		{
+			_MMU_write32<ARMCPU_ARM7>((nds.FW_ARM7BootCodeAddr + i), T1ReadLong(nds.FW_ARM7BootCode, i));
+		}
+
+		armcpu_init(&NDS_ARM9, nds.FW_ARM9BootCodeAddr);
+		armcpu_init(&NDS_ARM7, nds.FW_ARM7BootCodeAddr);
+		//armcpu_init(&NDS_ARM9, 0xFFFF0000);
+		//armcpu_init(&NDS_ARM7, 0x00000000);
+	}
+	else
+	{
+		src = header->ARM9src;
+		dst = header->ARM9cpy;
+
+		for(i = 0; i < (header->ARM9binSize>>2); ++i)
+		{
+			_MMU_write32<ARMCPU_ARM9>(dst, T1ReadLong(MMU.CART_ROM, src));
+			dst += 4;
+			src += 4;
+		}
+
+		src = header->ARM7src;
+		dst = header->ARM7cpy;
+
+		for(i = 0; i < (header->ARM7binSize>>2); ++i)
+		{
+			_MMU_write32<ARMCPU_ARM7>(dst, T1ReadLong(MMU.CART_ROM, src));
+			dst += 4;
+			src += 4;
+		}
+
+		armcpu_init(&NDS_ARM7, header->ARM7exe);
+		armcpu_init(&NDS_ARM9, header->ARM9exe);
+	}
+	
 	nds.ARM9Cycle = 0;
 	nds.ARM7Cycle = 0;
 	nds.cycles = 0;
@@ -565,6 +988,8 @@ void NDS_Reset( void)
 	nds.lignerendu = FALSE;
 	nds.touchX = nds.touchY = 0;
 	nds.isTouch = 0;
+	nds.debugConsole = CommonSettings.DebugConsole;
+	SetupMMU(nds.debugConsole);
 
 	_MMU_write16<ARMCPU_ARM9>(0x04000130, 0x3FF);
 	_MMU_write16<ARMCPU_ARM7>(0x04000130, 0x3FF);
@@ -592,71 +1017,15 @@ void NDS_Reset( void)
 	for (i = 0; i < ((0x170+0x90)/4); i++) {
 		_MMU_write32<ARMCPU_ARM9>(0x027FFE00+i*4, LE_TO_LOCAL_32(((u32*)MMU.CART_ROM)[i]));
 	}
+
+	// Write the header checksum to memory (the firmware needs it to see the cart)
+	_MMU_write16<ARMCPU_ARM9>(0x027FF808, T1ReadWord(MMU.CART_ROM, 0x15E));
+
 	MainScreen.offset = 0;
 	SubScreen.offset = 192;
 
 	//_MMU_write32[ARMCPU_ARM9](0x02007FFC, 0xE92D4030);
 
-	//ARM7 BIOS IRQ HANDLER
-#ifdef USE_REAL_BIOS
-	inf = fopen("BiosNds7.ROM","rb");
-#else
-	inf = NULL;
-#endif
-	if(inf) {
-		fread(MMU.ARM7_BIOS,1,16384,inf);
-		fclose(inf);
-		NDS_ARM7.swi_tab = 0;
-		INFO("ARM7 BIOS is loaded.\n");
-	} else {
-		NDS_ARM7.swi_tab = ARM7_swi_tab;
-		_MMU_write32<ARMCPU_ARM7>(0x00, 0xE25EF002);
-		_MMU_write32<ARMCPU_ARM7>(0x04, 0xEAFFFFFE);
-		_MMU_write32<ARMCPU_ARM7>(0x18, 0xEA000000);
-		_MMU_write32<ARMCPU_ARM7>(0x20, 0xE92D500F);
-		_MMU_write32<ARMCPU_ARM7>(0x24, 0xE3A00301);
-		_MMU_write32<ARMCPU_ARM7>(0x28, 0xE28FE000);
-		_MMU_write32<ARMCPU_ARM7>(0x2C, 0xE510F004);
-		_MMU_write32<ARMCPU_ARM7>(0x30, 0xE8BD500F);
-		_MMU_write32<ARMCPU_ARM7>(0x34, 0xE25EF004);
-	}
-
-	//ARM9 BIOS IRQ HANDLER
-#ifdef USE_REAL_BIOS
-	inf = fopen("BiosNds9.ROM","rb");
-#else
-	inf = NULL;
-	//memcpy(ARM9Mem.ARM9_BIOS + 0x20, gba_header_data_0x04, 156);
-#endif
-	if(inf) {
-		fread(ARM9Mem.ARM9_BIOS,1,4096,inf);
-		fclose(inf);
-		NDS_ARM9.swi_tab = 0;
-		INFO("ARM9 BIOS is loaded.\n");
-	} else {
-		NDS_ARM9.swi_tab = ARM9_swi_tab;
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0018, 0xEA000000);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0020, 0xE92D500F);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0024, 0xEE190F11);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0028, 0xE1A00620);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF002C, 0xE1A00600);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0030, 0xE2800C40);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0034, 0xE28FE000);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0038, 0xE510F004);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF003C, 0xE8BD500F);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0040, 0xE25EF004);
-	}
-
-
-
-
-	_MMU_write32<ARMCPU_ARM9>(0x0000004, 0xE3A0010E);
-	_MMU_write32<ARMCPU_ARM9>(0x0000008, 0xE3A01020);
-	// _MMU_write32<ARMCPU_ARM9>(0x000000C, 0xE1B02110);
-	_MMU_write32<ARMCPU_ARM9>(0x000000C, 0xE1B02040);
-	_MMU_write32<ARMCPU_ARM9>(0x0000010, 0xE3B02020);
-	// _MMU_write32<ARMCPU_ARM9>(0x0000010, 0xE2100202);
-
 	delete header;
 
 	GPU_Reset(MainScreen.gpu, 0);
@@ -665,6 +1034,17 @@ void NDS_Reset( void)
 	gpu3D->NDS_3D_Reset();
 	SPU_Reset();
 	cheatsSearchClose();
+
+#ifdef EXPERIMENTAL_WIFI
+	WIFI_Init(&wifiMac);
+
+	if(wifiMac.netEnabled) {
+		WIFI_SoftAP_Shutdown(&wifiMac);
+		WIFI_SoftAP_Init(&wifiMac);
+	}
+#endif
+
+	memcpy(FW_Mac, (MMU.fw.data + 0x36), 6);
 }
 
 int NDS_ImportSave(const char *filename)
@@ -784,7 +1164,15 @@ int NDS_WriteBMP_32bppBuffer(int width, int height, const void* buf, const char
 	elems_written += fwrite(&imageheader, 1, sizeof(imageheader), file);
 
 	for(int i=0;i<height;i++)
-		elems_written += fwrite((u8*)buf + (height-i-1)*width*4,1,imageheader.width*4,file);
+		for(int x=0;x<width;x++)
+		{
+			u8* pixel = (u8*)buf + (height-i-1)*width*4;
+			pixel += (x*4);
+			elems_written += fwrite(pixel+2,1,1,file);
+			elems_written += fwrite(pixel+1,1,1,file);
+			elems_written += fwrite(pixel+0,1,1,file);
+			elems_written += fwrite(pixel+3,1,1,file);
+		}
 	fclose(file);
 
 	return 1;
@@ -993,28 +1381,45 @@ int NDS_LoadFirmware(const char *filename)
 {
 	int i;
 	unsigned long size;
-	FILE *file;
+	//FILE *file;
 
-	if ((file = fopen(filename, "rb")) == NULL)
+	if ((MMU.fw.fp = fopen(filename, "rb+")) == NULL)
 		return -1;
 
-	fseek(file, 0, SEEK_END);
-	size = ftell(file);
-	fseek(file, 0, SEEK_SET);
+	fseek(MMU.fw.fp, 0, SEEK_END);
+	size = ftell(MMU.fw.fp);
+	fseek(MMU.fw.fp, 0, SEEK_SET);
 
 	if(size > MMU.fw.size)
 	{
-		fclose(file);
+		fclose(MMU.fw.fp);
+		fw_success = FALSE;
 		return -1;
 	}
 
-	i = fread(MMU.fw.data, size, 1, file);
-	fclose(file);
+	i = fread(MMU.fw.data, size, 1, MMU.fw.fp);
+	//fclose(file);
+
+	INFO("Firmware: decrypting NDS firmware %s...\n", filename);
+
+	if(decryptFirmware(MMU.fw.data) == FALSE)
+	{
+		INFO("Firmware: decryption failed.\n");
+		fw_success = FALSE;
+	}
+	else
+	{
+		INFO("Firmware: decryption successful.\n");
+		fw_success = TRUE;
+	}
 
 	return i;
 }
 
+void NDS_Sleep() { nds.sleeping = TRUE; }
+
 bool skipThisFrame = false;
+bool skipThisFrame3D = false;
 
 void NDS_SkipFrame(bool skip) { skipThisFrame = skip; }
 
@@ -1026,6 +1431,9 @@ u32 NDS_exec(s32 nb)
 {
 	int i, j;
 
+	//TODO - since NDS_exec is not necessarily called one frame at a time, this could be wrong.
+	LagFrameFlag=1;
+
 	nb += nds.cycles;//(nds.cycles>>26)<<26;
 
 	//increase this to execute more instructions in each batch (reducing overhead)
@@ -1057,6 +1465,9 @@ u32 NDS_exec(s32 nb)
 #endif
 				for (i = 0; i < INSTRUCTIONS_PER_BATCH && (!FORCE) && (execute); i++)
 				{
+					if(nds.sleeping) {
+						break;
+					} else
 					if(NDS_ARM9.waitIRQ) {
 						nds.ARM9Cycle += CYCLES_TO_WAIT_FOR_IRQ;
 						nds.idleCycles += CYCLES_TO_WAIT_FOR_IRQ;
@@ -1065,18 +1476,25 @@ u32 NDS_exec(s32 nb)
 						nds.ARM9Cycle += armcpu_exec<ARMCPU_ARM9>();
 				}
 #ifdef _WIN32
+#ifdef DEVELOPER
 				DisassemblerTools_Refresh(ARMCPU_ARM9);
+#endif
 #endif
 			}
 
 #ifdef EXPERIMENTAL_WIFI
+#if 0
 
 			if((nds.ARM7Cycle % 0x3F03) == 0)
 			{
-				/* 3F03 arm7 cyles = ~1usec */
+				/* 3F03 arm7 cycles = ~1usec */
 				WIFI_usTrigger(&wifiMac) ;
 			}
 #endif
+
+			int nds7old = nds.ARM7Cycle;
+#endif
+
 			if(nds.ARM7Cycle<=nds.cycles)
 			{
 #ifdef LOG_ARM7
@@ -1094,6 +1512,9 @@ u32 NDS_exec(s32 nb)
 #endif
 				for (i = 0; i < INSTRUCTIONS_PER_BATCH && (!FORCE) && (execute); i++)
 				{
+					if(nds.sleeping) {
+						break;
+					} else
 					if(NDS_ARM7.waitIRQ)
 					{
 						nds.ARM7Cycle += CYCLES_TO_WAIT_FOR_IRQ;
@@ -1103,11 +1524,32 @@ u32 NDS_exec(s32 nb)
 						nds.ARM7Cycle += (armcpu_exec<ARMCPU_ARM7>()<<1);
 				}
 #ifdef _WIN32
+#ifdef DEVELOPER
 				DisassemblerTools_Refresh(ARMCPU_ARM7);
+#endif
 #endif
 			}
+
+#ifdef EXPERIMENTAL_WIFI
+			/* TODO: the wifi stuff isn't actually clocked by the ARM7 clock, */
+			/* but by a 22 mhz oscillator. */
+			if(nds7old < nds.ARM7Cycle)
+			{
+				nds7old %= 67;
+				int nds7new = (nds.ARM7Cycle % 67);
+
+				if(nds7old > nds7new)
+				{
+					WIFI_usTrigger(&wifiMac);
+					WIFI_SoftAP_usTrigger(&wifiMac);
+				}
+			}
+#endif
 		}
 
+		if(!nds.sleeping)
+		{
+
 		nds.cycles = std::min(nds.ARM9Cycle,nds.ARM7Cycle);
 
 		//debug();
@@ -1129,6 +1571,8 @@ u32 NDS_exec(s32 nb)
 						GPU_ligne(&SubScreen, nds.VCount);
 					}
 
+					skipThisFrame3D = skipThisFrame;
+
 					if(MMU.DMAStartTime[0][0] == 2)
 						MMU_doDMA<ARMCPU_ARM9>(0);
 					if(MMU.DMAStartTime[0][1] == 2)
@@ -1204,7 +1648,7 @@ u32 NDS_exec(s32 nb)
 				nds.lignerendu = FALSE;
 				if(nds.VCount==192)
 				{
-					//osdA->update();	//================================= this is don't correct, need swap engine
+					//osdA->update();
 					gfx3d_VBlankSignal();
 
 					T1WriteWord(ARM9Mem.ARM9_REG, 4, T1ReadWord(ARM9Mem.ARM9_REG, 4) | 1);
@@ -1239,12 +1683,13 @@ u32 NDS_exec(s32 nb)
 				}
 				else if(nds.VCount==215)
 				{
-					gfx3d_VBlankEndSignal();
+					gfx3d_VBlankEndSignal(false);
 				}
 				else if(nds.VCount==263)
 				{
 					//osd->update();
-					//osdB->update();	//================================= this is don't correct, need swap engine
+					//osdB->update();
+					gpu_UpdateRender();
 
 					nds.nextHBlank = 3168;
 					nds.VCount = 0;
@@ -1767,14 +2212,53 @@ u32 NDS_exec(s32 nb)
 			}
 		}
 
+		} // if(!nds.sleeping)
+		else
+		{
+			gpu_UpdateRender();
+			if((MMU.reg_IE[1] & MMU.reg_IF[1]) & (1<<22))
+			{
+				nds.sleeping = FALSE;
+			}
+			break;
+		}
+	}
 
-
-
+	if(LagFrameFlag)
+	{
+		lagframecounter++;
+		TotalLagFrames++;
+	}
+	else 
+	{
+		lastLag = lagframecounter;
+		lagframecounter = 0;
 	}
 
 	return nds.cycles;
 }
 
+static std::string MakeInputDisplayString(u16 pad, const std::string* Buttons, int count) {
+    std::string s;
+    for (int x = 0; x < count; x++) {
+        if (pad & (1 << x))
+            s.append(Buttons[x].size(), ' '); 
+        else
+            s += Buttons[x];
+    }
+    return s;
+}
+
+static std::string MakeInputDisplayString(u16 pad, u16 padExt) {
+    const std::string Buttons[] = {"A", "B", "Sl", "St", "R", "L", "U", "D", "Rs", "Ls"};
+    const std::string Ext[] = {"X", "Y"};
+
+    std::string s = MakeInputDisplayString(pad, Ext, ARRAY_SIZE(Ext));
+    s += MakeInputDisplayString(padExt, Buttons, ARRAY_SIZE(Buttons));
+
+    return s;
+}
+
 void NDS_setPadFromMovie(u16 pad)
 {
 #define FIX(b,n) (((pad>>n)&1)!=0)
@@ -1837,12 +2321,12 @@ void NDS_setPad(bool R,bool L,bool D,bool U,bool T,bool S,bool B,bool A,bool Y,b
 		LidClosed = (!LidClosed) & 0x01;
 		if (!LidClosed)
 		{
-			SPU_Pause(FALSE);
+		//	SPU_Pause(FALSE);
 			NDS_makeARM7Int(22);
 
 		}
-		else
-			SPU_Pause(TRUE);
+		//else
+			//SPU_Pause(TRUE);
 
 		countLid = 30;
 	}
@@ -1861,6 +2345,8 @@ void NDS_setPad(bool R,bool L,bool D,bool U,bool T,bool S,bool B,bool A,bool Y,b
 
 	((u16 *)MMU.ARM7_REG)[0x136>>1] = (u16)padExt;
 
+	InputDisplayString=MakeInputDisplayString(padExt, pad);
+
 	//put into the format we want for the movie system
 	//RLDUTSBAYXWEGF
 #undef FIX
