@@ -312,6 +312,12 @@ TEMPLATE static u32 wait4IRQ()
      return 1;
 }
 
+TEMPLATE static u32 sleep()
+{
+	_MMU_write08(cpu->proc_ID, 0x04000301, 0xC0);
+	return 1;
+}
+
 TEMPLATE static u32 divide()
 {
      s32 num = (s32)cpu->R[0];
@@ -332,6 +338,8 @@ TEMPLATE static u32 copy()
      u32 dst = cpu->R[1];
      u32 cnt = cpu->R[2];
 
+	 //INFO("swi copy from %08X to %08X, cnt=%08X\n", src, dst, cnt);
+
      switch(BIT26(cnt))
      {
           case 0:
@@ -402,6 +410,8 @@ TEMPLATE static u32 fastCopy()
      u32 dst = cpu->R[1] & 0xFFFFFFFC;
      u32 cnt = cpu->R[2];
 
+	 //INFO("swi fastcopy from %08X to %08X, cnt=%08X\n", src, dst, cnt);
+
      switch(BIT24(cnt))
      {
           case 0:
@@ -442,6 +452,8 @@ TEMPLATE static u32 LZ77UnCompVram()
   u32 header = _MMU_read32(cpu->proc_ID,source);
   source += 4;
 
+  //INFO("swi lz77uncompvram\n");
+
   if(((source & 0xe000000) == 0) ||
      ((source + ((header >> 8) & 0x1fffff)) & 0xe000000) == 0)
     return 0;    
@@ -530,6 +542,8 @@ TEMPLATE static u32 LZ77UnCompWram()
   u32 header = _MMU_read32(cpu->proc_ID, source);
   source += 4;
 
+  //INFO("swi lz77uncompwram\n");
+
   if(((source & 0xe000000) == 0) ||
      ((source + ((header >> 8) & 0x1fffff)) & 0xe000000) == 0)
     return 0;  
@@ -589,6 +603,8 @@ TEMPLATE static u32 RLUnCompVram()
   u32 header = _MMU_read32(cpu->proc_ID, source);
   source += 4;
 
+  //INFO("swi rluncompvram\n");
+
   if(((source & 0xe000000) == 0) ||
      ((source + ((header >> 8) & 0x1fffff)) & 0xe000000) == 0)
     return 0;  
@@ -652,6 +668,8 @@ TEMPLATE static u32 RLUnCompWram()
   u32 header = _MMU_read32(cpu->proc_ID, source);
   source += 4;
 
+  //INFO("swi rluncompwram\n");
+
   if(((source & 0xe000000) == 0) ||
      ((source + ((header >> 8) & 0x1fffff)) & 0xe000000) == 0)
     return 0;  
@@ -697,6 +715,8 @@ TEMPLATE static u32 UnCompHuffman()
   header = _MMU_read08(cpu->proc_ID, source);
   source += 4;
 
+  //INFO("swi uncomphuffman\n");
+
   if(((source & 0xe000000) == 0) ||
      ((source + ((header >> 8) & 0x1fffff)) & 0xe000000) == 0)
     return 0;  
@@ -836,6 +856,8 @@ TEMPLATE static u32 BitUnPack()
   source = cpu->R[0];
   dest = cpu->R[1];
   header = cpu->R[2];
+
+  //INFO("swi bitunpack\n");
   
   len = _MMU_read16(cpu->proc_ID, header);
   // check address
@@ -892,6 +914,8 @@ TEMPLATE static u32 Diff8bitUnFilterWram()
   header = _MMU_read08(cpu->proc_ID, source);
   source += 4;
 
+  //INFO("swi diff8bitunfilterwram\n");
+
   if(((source & 0xe000000) == 0) ||
      (( (source + ((header >> 8) & 0x1fffff)) & 0xe000000) == 0))
     return 0;  
@@ -920,6 +944,8 @@ TEMPLATE static u32 Diff16bitUnFilter()
   source = cpu->R[0];
   dest = cpu->R[1];
 
+  //INFO("swi diff16bitunfilter\n");
+
   header = _MMU_read08(cpu->proc_ID, source);
   source += 4;
 
@@ -1006,6 +1032,17 @@ TEMPLATE static u32 getCRC16()
   return 1;
 }
 
+TEMPLATE static u32 isDebugger()
+{
+	//gbatek has additional specifications which are not emulated here
+#ifdef DEVELOPER
+	cpu->R[0] = 1;
+#else
+	cpu->R[0] = 0;
+#endif
+	return 1;
+}
+
 TEMPLATE static u32 SoundBias()
 {
      u32 current = SPU_ReadLong(0x4000504);
@@ -1016,6 +1053,14 @@ TEMPLATE static u32 SoundBias()
      return cpu->R[1];
 }
 
+TEMPLATE static u32 getBootProcs()
+{
+	cpu->R[0] = 0x00000A2E;
+	cpu->R[1] = 0x00002C3C;
+	cpu->R[3] = 0x000005FF;
+	return 1;
+}
+
 u32 (* ARM9_swi_tab[32])()={
          bios_nop<ARMCPU_ARM9>,             // 0x00
          bios_nop<ARMCPU_ARM9>,             // 0x01
@@ -1032,7 +1077,7 @@ u32 (* ARM9_swi_tab[32])()={
          fastCopy<ARMCPU_ARM9>,             // 0x0C
          bios_sqrt<ARMCPU_ARM9>,            // 0x0D
          getCRC16<ARMCPU_ARM9>,             // 0x0E
-         bios_nop<ARMCPU_ARM9>,             // 0x0F
+         isDebugger<ARMCPU_ARM9>,           // 0x0F
          BitUnPack<ARMCPU_ARM9>,            // 0x10
          LZ77UnCompWram<ARMCPU_ARM9>,       // 0x11
          LZ77UnCompVram<ARMCPU_ARM9>,       // 0x12
@@ -1059,7 +1104,7 @@ u32 (* ARM7_swi_tab[32])()={
          intrWaitARM<ARMCPU_ARM7>,          // 0x04
          waitVBlankARM<ARMCPU_ARM7>,        // 0x05
          wait4IRQ<ARMCPU_ARM7>,             // 0x06
-         wait4IRQ<ARMCPU_ARM7>,             // 0x07
+         sleep<ARMCPU_ARM7>,                // 0x07
          SoundBias<ARMCPU_ARM7>,            // 0x08
          divide<ARMCPU_ARM7>,               // 0x09
          bios_nop<ARMCPU_ARM7>,             // 0x0A
@@ -1067,7 +1112,7 @@ u32 (* ARM7_swi_tab[32])()={
          fastCopy<ARMCPU_ARM7>,             // 0x0C
          bios_sqrt<ARMCPU_ARM7>,            // 0x0D
          getCRC16<ARMCPU_ARM7>,             // 0x0E
-         bios_nop<ARMCPU_ARM7>,             // 0x0F
+		 isDebugger<ARMCPU_ARM7>,           // 0x0F
          BitUnPack<ARMCPU_ARM7>,            // 0x10
          LZ77UnCompWram<ARMCPU_ARM7>,       // 0x11
          LZ77UnCompVram<ARMCPU_ARM7>,       // 0x12
@@ -1081,7 +1126,7 @@ u32 (* ARM7_swi_tab[32])()={
          getSineTab<ARMCPU_ARM7>,           // 0x1A
          getPitchTab<ARMCPU_ARM7>,          // 0x1B
          getVolumeTab<ARMCPU_ARM7>,         // 0x1C
-         bios_nop<ARMCPU_ARM7>,             // 0x1D
+         getBootProcs<ARMCPU_ARM7>,         // 0x1D
          bios_nop<ARMCPU_ARM7>,             // 0x1E
          setHaltCR<ARMCPU_ARM7>,            // 0x1F
 };
