@@ -52,12 +52,17 @@
 #include "../sndsdl.h"
 #include "../ctrlssdl.h"
 #include "../render3D.h"
+#include "../rasterize.h"
+#include "../saves.h"
 #ifdef GDB_STUB
 #include "../gdbstub.h"
 #endif
 
 volatile BOOL execute = FALSE;
 
+int savetype=MC_TYPE_AUTODETECT;
+u32 savesize=1;
+
 static float nds_screen_size_ratio = 1.0f;
 
 #define DISPLAY_FPS 1
@@ -82,33 +87,51 @@ SoundInterface_struct *SNDCoreList[] = {
 };
 
 GPU3DInterface *core3DList[] = {
-&gpu3DNull
+&gpu3DNull,
+&gpu3DRasterize,
+NULL
+};
+
+const char * save_type_names[] = {
+    "Autodetect",
+    "EEPROM 4kbit",
+    "EEPROM 64kbit",
+    "EEPROM 512kbit",
+    "FRAM 256kbit",
+    "FLASH 2mbit",
+    "FLASH 4mbit",
+    NULL
 };
 
 
 /* Our keyboard config is different because of the directional keys */
 const u16 cli_kb_cfg[NB_KEYS] =
-  { SDLK_c,         // A
-    SDLK_x,         // B
-    SDLK_BACKSPACE, // select
+  { 
+    SDLK_x,         // A
+    SDLK_z,         // B
+    SDLK_RSHIFT,    // select
     SDLK_RETURN,    // start
     SDLK_RIGHT,     // Right
     SDLK_LEFT,      // Left
     SDLK_UP,        // Up
     SDLK_DOWN,      // Down
-    SDLK_e,         // R
-    SDLK_w,         // L
-    SDLK_d,         // X
-    SDLK_s,         // Y
+    SDLK_w,         // R
+    SDLK_q,         // L
+    SDLK_s,         // X
+    SDLK_a,         // Y
     SDLK_p,         // DEBUG
     SDLK_o          // BOOST
   };
 
 struct my_config {
+  int load_slot;
   u16 arm9_gdb_port;
   u16 arm7_gdb_port;
 
   int disable_sound;
+  int engine_3d;
+
+  int savetype;
 
 #ifdef INCLUDE_OPENGL_2D
   int opengl_2d;
@@ -124,6 +147,8 @@ struct my_config {
 
 static void
 init_config( struct my_config *config) {
+  config->load_slot = 0;
+
   config->arm9_gdb_port = 0;
   config->arm7_gdb_port = 0;
 
@@ -135,6 +160,9 @@ init_config( struct my_config *config) {
 
   config->cflash_disk_image_file = NULL;
 
+  config->engine_3d = 1;
+  config->savetype = 0;
+
 #ifdef INCLUDE_OPENGL_2D
   config->opengl_2d = 0;
   config->soft_colour_convert = 0;
@@ -154,16 +182,26 @@ fill_config( struct my_config *config,
 
   for ( i = 1; i < argc && good_args; i++) {
     if ( strcmp( argv[i], "--help") == 0) {
-      printf( "USAGE: %s <nds-file>\n", argv[0]);
+      printf( "USAGE: %s [options] <nds-file>\n", argv[0]);
       printf( "OPTIONS:\n");
-      printf( "   --disable-sound         Disables the sound emulation\n");
+      printf( "   --load-slot=NUM     Loads savegame from slot NUM\n");
+      printf( "   --disable-sound     Disables the sound emulation\n");
       printf( "   --disable-limiter   Disables the 60 fps limiter\n");
+      printf( "   --3d-engine=ENGINE  Select 3d rendering engine, available ENGINES:\n");
+      printf( "                         0 = 3d disabled\n");
+      printf( "                         1 = internal desmume software rasterizer (default)\n");
 #ifdef INCLUDE_OPENGL_2D
       printf( "   --opengl-2d         Enables using OpenGL for screen rendering\n");
-      printf( "    --soft-convert     Use software colour conversion during OpenGL\n");
+      printf( "   --soft-convert      Use software colour conversion during OpenGL\n");
       printf( "                       screen rendering. May produce better or worse\n");
       printf( "                       frame rates depending on hardware.\n");
 #endif
+      printf( "\n");
+      printf( "   --save-type=TYPE    Select savetype from the following:\n");
+      for(int jj = 0; save_type_names[jj] != NULL; jj++){
+          printf("                         %d = %s\n",jj,save_type_names[jj]);
+      }
+
       printf( "\n");
       printf( "   --fwlang=LANG       Set the language in the firmware, LANG as follows:\n");
       printf( "                         0 = Japanese\n");
@@ -174,22 +212,29 @@ fill_config( struct my_config *config,
       printf( "                         5 = Spanish\n");
       printf( "\n");
 #ifdef GDB_STUB
-      printf( "   --arm9gdb=PORT_NUM      Enable the ARM9 GDB stub on the given port\n");
-      printf( "   --arm7gdb=PORT_NUM      Enable the ARM7 GDB stub on the given port\n");
+      printf( "   --arm9gdb=PORT_NUM  Enable the ARM9 GDB stub on the given port\n");
+      printf( "   --arm7gdb=PORT_NUM  Enable the ARM7 GDB stub on the given port\n");
 #endif
       //printf( "   --sticky                Enable sticky keys and stylus\n");
       printf( "\n");
       printf( "   --cflash=PATH_TO_DISK_IMAGE\n");
       printf( "                       Enable disk image GBAMP compact flash emulation\n");
       printf( "\n");
-      printf( "   --help                  Display this message\n");
-      printf( "   --version               Display the version\n");
+      printf( "   --help              Display this message\n");
+      printf( "   --version           Display the version\n");
       good_args = 0;
     }
     else if ( strcmp( argv[i], "--version") == 0) {
       printf( "%s\n", VERSION);
       good_args = 0;
     }
+    else if ( strncmp( argv[i], "--load-slot=", 12) == 0) {
+      long slot = strtol( &argv[i][12], NULL, 10 );
+      if(slot >= 0 && slot <= 10)
+        config->load_slot = slot;
+      else
+        printf("Invalid slot number %ld\n", slot);
+    }
     else if ( strcmp( argv[i], "--disable-sound") == 0) {
       config->disable_sound = 1;
     }
@@ -204,6 +249,31 @@ fill_config( struct my_config *config,
     else if ( strcmp( argv[i], "--disable-limiter") == 0) {
       config->disable_limiter = 1;
     }
+    else if ( strncmp( argv[i], "--3d-engine=", 12) == 0) {
+      char *end_char;
+      int engine = strtoul( &argv[i][12], &end_char, 10);
+
+      if ( engine == 0 || engine == 1) {
+        config->engine_3d = engine;
+      }
+      else {
+        fprintf( stderr, "3d engine can be 0 or 1\n");
+        good_args = 0;
+      }
+    }
+    else if ( strncmp( argv[i], "--save-type=", 12) == 0) {
+      char *end_char;
+      int savetype = strtoul( &argv[i][12], &end_char, 10);
+      int last = sizeof(save_type_names)/sizeof(const char * )-2; // NULL terminator, 0-based 
+
+      if ( savetype >= 0 && savetype <= last) {
+        config->savetype = savetype;
+      }
+      else {
+        fprintf( stderr, "savetype can be 0-%d\n",last);
+        good_args = 0;
+      }
+    }
     else if ( strncmp( argv[i], "--fwlang=", 9) == 0) {
       char *end_char;
       int lang = strtoul( &argv[i][9], &end_char, 10);
@@ -271,7 +341,7 @@ fill_config( struct my_config *config,
   }
 
   if ( print_usage) {
-    fprintf( stderr, "USAGE: %s <nds-file>\n", argv[0]);
+    fprintf( stderr, "USAGE: %s <nds-file>\n  %s --help for more info\n", argv[0], argv[0]);
   }
 
   return good_args;
@@ -632,7 +702,11 @@ int main(int argc, char ** argv) {
     SPU_ChangeSoundCore(SNDCORE_SDL, 735 * 4);
   }
 
-  if (NDS_LoadROM( my_config.nds_file, MC_TYPE_AUTODETECT, 1, my_config.cflash_disk_image_file) < 0) {
+  NDS_3D_ChangeCore(my_config.engine_3d);
+
+  mmu_select_savetype(my_config.savetype, &savetype, &savesize);
+
+  if (NDS_LoadROM( my_config.nds_file, savetype, savesize, my_config.cflash_disk_image_file) < 0) {
     fprintf(stderr, "error while loading %s\n", my_config.nds_file);
     exit(-1);
   }
@@ -758,7 +832,9 @@ int main(int argc, char ** argv) {
     }
   }
 
-
+  if(my_config.load_slot){
+    loadstate_slot(my_config.load_slot);
+  }
 
   while(!sdl_quit) {
     /* Look for queued events and update keypad status */
