@@ -21,6 +21,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <string.h>
 #include "common.h"
 
 u8	gba_header_data_0x04[156] = {
@@ -86,3 +87,23 @@ u8 reverseBitsInByte(u8 x)
 
 	return h;
 }
+
+void removeCR(char *buf)
+{
+	int l=strlen(buf);
+	for (int i=0; i < l; i++)
+	{
+		if (buf[i]==0x0A) buf[i]=0;
+		if (buf[i]==0x0D) buf[i]=0;
+	}
+}
+
+u32 strlen_ws(char *buf)		// length without last spaces
+{
+	if (!strlen(buf)) return 0;
+	for (int i=strlen(buf); i>0; i--)
+	{
+		if (buf[i]!=32) return (i-1);		// space
+	}
+	return 0;
+}
