@@ -31,9 +31,22 @@ extern u8	gba_header_data_0x04[156];
 
 #ifdef WIN32
 
-	#define _WINSOCKAPI_
+	#include <winsock2.h>
 	#include <windows.h>
 
+	#define IDT_VIEW_DISASM7						50001
+	#define IDT_VIEW_DISASM9                		50002
+	#define IDT_VIEW_MEM7                   		50003
+	#define IDT_VIEW_MEM9                   		50004
+	#define IDT_VIEW_IOREG                  		50005
+	#define IDT_VIEW_PAL                    		50006
+	#define IDT_VIEW_TILE                   		50007
+	#define IDT_VIEW_MAP                    		50008
+	#define IDT_VIEW_OAM                    		50009
+	#define IDT_VIEW_MATRIX                 		50010
+	#define IDT_VIEW_LIGHTS                 		50011
+	#define IDM_EXEC								50112
+
 	#define CLASSNAME "DeSmuME"
 
 	extern HINSTANCE hAppInst;
@@ -45,19 +58,15 @@ extern u8	gba_header_data_0x04[156];
 	extern void WritePrivateProfileInt(char* appname, char* keyname, int val, char* file);
 
 	#define EXPERIMENTAL_GBASLOT 1
-	// this is experimental and only tested for windows port 
-	// (I can`t test on another ports) 
-	// 
-	// About GBA game in slot (only for use together with NDS + GBA): 
-	// in real BIOS 9 at offset 0x0020 placed compressed logo 
-	// for comparing with logo in header GBA cartridge. 
-	// so, GBA game in slot work now.
-	// Later need make loading this table in BIOS memory (from gba_header_data_0x04)
-	//
-	// in windows ports:
-	// i added in menu "Emulation" submenu "GBA slot"
+#else		// non Windows
+
+#define sscanf_s sscanf
+
 #endif
 
-extern u8 reverseBitsInByte(u8 x);
+extern u8	reverseBitsInByte(u8 x);
+extern void	removeCR(char *buf);
+extern u32	strlen_ws(char *buf);
 	
-#endif
\ No newline at end of file
+#endif
+
