@@ -26,6 +26,7 @@
 // plugin responsible only for drawing primitives.
 
 #include <algorithm>
+#include <assert.h>
 #include <math.h>
 #include "armcpu.h"
 #include "debug.h"
@@ -45,6 +46,7 @@ GFX3D gfx3d;
 //tables that are provided to anyone
 CACHE_ALIGN u32 color_15bit_to_24bit_reverse[32768];
 CACHE_ALIGN u32 color_15bit_to_24bit[32768];
+CACHE_ALIGN u16 color_15bit_to_16bit_reverse[32768];
 CACHE_ALIGN u8 mixTable555[32][32][32];
 
 //is this a crazy idea? this table spreads 5 bits evenly over 31 from exactly 0 to INT_MAX
@@ -59,6 +61,13 @@ CACHE_ALIGN const int material_5bit_to_31bit[] = {
 	0x739CE739, 0x77BDEF7B, 0x7BDEF7BD, 0x7FFFFFFF
 };
 
+CACHE_ALIGN const u8 material_5bit_to_6bit[] = {
+	0x00, 0x02, 0x04, 0x06, 0x08, 0x0A, 0x0C, 0x0E,
+	0x10, 0x12, 0x14, 0x16, 0x19, 0x1A, 0x1C, 0x1E,
+	0x21, 0x23, 0x25, 0x27, 0x29, 0x2B, 0x2D, 0x2F,
+	0x31, 0x33, 0x35, 0x37, 0x39, 0x3B, 0x3D, 0x3F
+};
+
 CACHE_ALIGN const u8 material_5bit_to_8bit[] = {
 	0x00, 0x08, 0x10, 0x18, 0x21, 0x29, 0x31, 0x39,
 	0x42, 0x4A, 0x52, 0x5A, 0x63, 0x6B, 0x73, 0x7B,
@@ -121,12 +130,14 @@ static u8 MM3x3ind = 0;
 static CACHE_ALIGN float	coord[4] = {0.0, 0.0, 0.0, 0.0};
 static char		coordind = 0;
 static u32 vtxFormat;
+static BOOL inBegin = FALSE;
 
 // Data for basic transforms
 static CACHE_ALIGN float	trans[4] = {0.0, 0.0, 0.0, 0.0};
 static int		transind = 0;
 static CACHE_ALIGN float	scale[4] = {0.0, 0.0, 0.0, 0.0};
 static int		scaleind = 0;
+static u32 viewport;
 
 //various other registers
 static float _t=0, _s=0;
@@ -138,10 +149,11 @@ static u32 clInd2 = 0;
 #endif
 static u32 BTind = 0;
 static u32 PTind = 0;
+//static CACHE_ALIGN float BTcoords[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
 static CACHE_ALIGN float PTcoords[4] = {0.0, 0.0, 0.0, 1.0};
 
 //raw ds format poly attributes
-static u32 polyAttr=0,textureFormat=0, texturePalette=0;
+static u32 polyAttr=0,textureFormat=0, texturePalette=0, polyAttrPending=0;
 
 //the current vertex color, 5bit values
 static int colorRGB[4] = { 31,31,31,31 };
@@ -180,8 +192,6 @@ POLYLIST polylists[2];
 POLYLIST* polylist = &polylists[0];
 VERTLIST vertlists[2];
 VERTLIST* vertlist = &vertlists[0];
-//PROJLIST projlists[2];
-//PROJLIST* projlist = &projlists[0];
 
 int listTwiddle = 1;
 int triStripToggle;
@@ -192,6 +202,8 @@ struct tmpVertInfo {
 	int count;
 	//indices to the main vert list
 	int map[4];
+	//indicates that the first poly in a list has been completed
+	BOOL first;
 } tempVertInfo;
 
 
@@ -200,10 +212,8 @@ static void twiddleLists() {
 	listTwiddle &= 1;
 	polylist = &polylists[listTwiddle];
 	vertlist = &vertlists[listTwiddle];
-//	projlist = &projlists[listTwiddle];
 	polylist->count = 0;
 	vertlist->count = 0;
-//	projlist->count = 0;
 }
 
 static BOOL flushPending = FALSE;
@@ -222,7 +232,10 @@ static void makeTables() {
 	#define RGB15TO24_BITLOGIC_REVERSE(col) ( (material_5bit_to_8bit[(col)&0x1F]<<16) | (material_5bit_to_8bit[((col)>>5)&0x1F]<<8) | material_5bit_to_8bit[((col)>>10)&0x1F] )
 
 	for(int i=0;i<32768;i++)
+	{
 		color_15bit_to_24bit_reverse[i] = RGB15TO24_BITLOGIC_REVERSE((u16)i);
+		color_15bit_to_16bit_reverse[i] = (((i & 0x001F) << 11) | (material_5bit_to_6bit[(i & 0x03E0) >> 5] << 5) | ((i & 0x7C00) >> 10));
+	}
 
 	for (int i = 0; i < 65536; i++)
 		float16table[i] = fix2float((signed short)i);
@@ -283,29 +296,32 @@ void gfx3d_reset()
 	_s=0;
 	last_t = 0;
 	last_s = 0;
+	viewport = 0xBFFF0000;
 	
 	GFX_FIFOclear();
 }
 
-void gfx3d_glViewPort(unsigned long v)
+void gfx3d_glViewPort(u32 v)
 {
-	//zero: NHerve messed with this in mod2 and mod3, but im still not sure its perfect. need to research this.
-	gfx3d.viewport.x = (v&0xFF);
-	gfx3d.viewport.y = (v&0xFF);
-	gfx3d.viewport.width = (((v>>16)&0xFF)+1)-(v&0xFF);
-	gfx3d.viewport.height = ((v>>24)+1)-((v>>8)&0xFF);
+	viewport = v;
+}
+
+void VIEWPORT::decode(u32 v) 
+{
+	x = (v&0xFF);
+	y = std::min(191,(int)(((v>>8)&0xFF)));
+	width = (((v>>16)&0xFF)+1)-(v&0xFF);
+	height = ((v>>24)+1)-((v>>8)&0xFF);
 }
 
 
-void gfx3d_glClearColor(unsigned long v)
+void gfx3d_glClearColor(u32 v)
 {
-	gfx3d.clearColor[0] = ((float)(v&0x1F))/31.0f;
-	gfx3d.clearColor[1] = ((float)((v>>5)&0x1F))/31.0f;
-	gfx3d.clearColor[2] = ((float)((v>>10)&0x1F))/31.0f;
-	gfx3d.clearColor[3] = ((float)((v>>16)&0x1F))/31.0f;
+	gfx3d.clearColor = v;
+
 }
 
-void gfx3d_glFogColor(unsigned long v)
+void gfx3d_glFogColor(u32 v)
 {
 	gfx3d.fogColor[0] = ((float)((v    )&0x1F))/31.0f;
 	gfx3d.fogColor[1] = ((float)((v>> 5)&0x1F))/31.0f;
@@ -313,20 +329,19 @@ void gfx3d_glFogColor(unsigned long v)
 	gfx3d.fogColor[3] = ((float)((v>>16)&0x1F))/31.0f;
 }
 
-void gfx3d_glFogOffset (unsigned long v)
+void gfx3d_glFogOffset (u32 v)
 {
 	gfx3d.fogOffset = (float)(v&0xffff);
 }
 
-void gfx3d_glClearDepth(unsigned long v)
+void gfx3d_glClearDepth(u32 v)
 {
-	//Thanks to NHerve
+	//formula from http://nocash.emubase.de/gbatek.htm#ds3drearplane
 	v &= 0x7FFFF;
-	u32 depth24b = (v*0x200)+((v+1)/0x8000)*0x01FF;
-	gfx3d.clearDepth = depth24b / ((float)(1<<24));
+	gfx3d.clearDepth = (v*0x200)+((v+1)/0x8000)*0x01FF;
 }
 
-void gfx3d_glMatrixMode(unsigned long v)
+void gfx3d_glMatrixMode(u32 v)
 {
 	mode = (v&3);
 }
@@ -372,7 +387,7 @@ BOOL gfx3d_glLoadMatrix4x3(signed long v)
 	return TRUE;
 }
 
-void gfx3d_glStoreMatrix(unsigned long v)
+void gfx3d_glStoreMatrix(u32 v)
 {
 	//this command always works on both pos and vector when either pos or pos-vector are the current mtx mode
 	short mymode = (mode==1?2:mode);
@@ -386,7 +401,7 @@ void gfx3d_glStoreMatrix(unsigned long v)
 		MatrixStackLoadMatrix (&mtxStack[1], v&31, mtxCurrent[1]);
 }
 
-void gfx3d_glRestoreMatrix(unsigned long v)
+void gfx3d_glRestoreMatrix(u32 v)
 {
 	//this command always works on both pos and vector when either pos or pos-vector are the current mtx mode
 	short mymode = (mode==1?2:mode);
@@ -548,19 +563,39 @@ BOOL gfx3d_glMultMatrix4x4(signed long v)
 	return TRUE;
 }
 
-void gfx3d_glBegin(unsigned long v)
+static void gfx3d_glPolygonAttrib_cache()
 {
+	// Light enable/disable
+	lightMask = (polyAttr&0xF);
+
+	// texture environment
+	envMode = (polyAttr&0x30)>>4;
+
+	// back face culling
+	cullingMask = (polyAttr>>6)&3;
+
+	// Alpha value, actually not well handled, 0 should be wireframe
+	colorRGB[3] = colorAlpha = ((polyAttr>>16)&0x1F);
+}
+
+void gfx3d_glBegin(u32 v)
+{
+	inBegin = TRUE;
 	vtxFormat = v&0x03;
 	triStripToggle = 0;
 	tempVertInfo.count = 0;
+	tempVertInfo.first = true;
+	polyAttr = polyAttrPending;
+	gfx3d_glPolygonAttrib_cache();
 }
 
 void gfx3d_glEnd(void)
 {
+	inBegin = FALSE;
 	tempVertInfo.count = 0;
 }
 
-void gfx3d_glColor3b(unsigned long v)
+void gfx3d_glColor3b(u32 v)
 {
 	colorRGB[0] = (v&0x1F);
 	colorRGB[1] = ((v>>5)&0x1F);
@@ -599,18 +634,19 @@ static void SetVertex()
 	//apply projection matrix
 	MatrixMultVec4x4 (mtxCurrent[0], coordTransformed);
 
-	////perspective division
-	//coordTransformed[0] = (coordTransformed[0] + coordTransformed[3]) / 2 / coordTransformed[3];
-	//coordTransformed[1] = (coordTransformed[1] + coordTransformed[3]) / 2 / coordTransformed[3];
-	//coordTransformed[2] = (coordTransformed[2] + coordTransformed[3]) / 2 / coordTransformed[3];
-	//coordTransformed[3] = 1;
-
 	//TODO - culling should be done here.
-	//TODO - viewport transform
+	//TODO - viewport transform?
+
+	int continuation = 0;
+	if(vtxFormat==2 && !tempVertInfo.first)
+		continuation = 2;
+	else if(vtxFormat==3 && !tempVertInfo.first)
+		continuation = 2;
+
 
 	//record the vertex
 	//VERT &vert = tempVertList.list[tempVertList.count];
-	VERT &vert = vertlist->list[vertlist->count + tempVertInfo.count];
+	VERT &vert = vertlist->list[vertlist->count + tempVertInfo.count - continuation];
 
 	vert.texcoord[0] = last_s;
 	vert.texcoord[1] = last_t;
@@ -621,9 +657,7 @@ static void SetVertex()
 	vert.color[0] = colorRGB[0];
 	vert.color[1] = colorRGB[1];
 	vert.color[2] = colorRGB[2];
-	vert.color[3] = colorRGB[3];
-	vert.depth = 0x7FFF * coordTransformed[2];
-	tempVertInfo.map[tempVertInfo.count] = vertlist->count + tempVertInfo.count;
+	tempVertInfo.map[tempVertInfo.count] = vertlist->count + tempVertInfo.count - continuation;
 	tempVertInfo.count++;
 
 	//possibly complete a polygon
@@ -664,13 +698,17 @@ static void SetVertex()
 				polylist->list[polylist->count].type = 3;
 
 				if(triStripToggle)
-					tempVertInfo.map[1] = vertlist->count+2;
+					tempVertInfo.map[1] = vertlist->count+2-continuation;
 				else
-					tempVertInfo.map[0] = vertlist->count+2;
+					tempVertInfo.map[0] = vertlist->count+2-continuation;
 				
-				vertlist->count+=3;
+				if(tempVertInfo.first)
+					vertlist->count+=3;
+				else
+					vertlist->count+=1;
 
 				triStripToggle ^= 1;
+				tempVertInfo.first = false;
 				tempVertInfo.count = 2;
 				break;
 			case 3: //GL_QUAD_STRIP
@@ -682,9 +720,12 @@ static void SetVertex()
 				SUBMITVERTEX(2,3);
 				SUBMITVERTEX(3,2);
 				polylist->list[polylist->count].type = 4;
-				tempVertInfo.map[0] = vertlist->count+2;
-				tempVertInfo.map[1] = vertlist->count+3;
-				vertlist->count+=4;
+				tempVertInfo.map[0] = vertlist->count+2-continuation;
+				tempVertInfo.map[1] = vertlist->count+3-continuation;
+				if(tempVertInfo.first)
+					vertlist->count+=4;
+				else vertlist->count+=2;
+				tempVertInfo.first = false;
 				tempVertInfo.count = 2;
 				break;
 		}
@@ -692,31 +733,11 @@ static void SetVertex()
 		if(completed)
 		{
 			POLY &poly = polylist->list[polylist->count];
-			//todo - dont overrun proj list
-			
-			//see if the last entry in the proj list matches the current matrix, if there is one.
-		/*	if(projlist->count != 0 && 
-				//here is an example of something that does not work.
-				//(for a speed hack, we consider the matrices different if the first element differs)
-				//mtxCurrent[0][0] == projlist->projMatrix[projlist->count-1][0]
-				
-				//here is what we must do: make sure the matrices are identical
-				!MatrixCompare(mtxCurrent[0],projlist->projMatrix[projlist->count-1])
-				)
-			{
-				//it matches. use it
-				poly.projIndex = projlist->count-1;
-			}
-			else
-			{
-				MatrixCopy(projlist->projMatrix[projlist->count],mtxCurrent[0]);
-				poly.projIndex = projlist->count;
-				projlist->count++;
-			}*/
 
 			poly.polyAttr = polyAttr;
 			poly.texParam = textureFormat;
 			poly.texPalette = texturePalette;
+			poly.viewport = viewport;
 			polylist->count++;
 		}
 	}
@@ -740,7 +761,7 @@ BOOL gfx3d_glVertex16b(unsigned int v)
 	return TRUE;
 }
 
-void gfx3d_glVertex10b(unsigned long v)
+void gfx3d_glVertex10b(u32 v)
 {
 	coord[0]		= float10Table[v&1023];
 	coord[1]		= float10Table[(v>>10)&1023];
@@ -757,7 +778,7 @@ void gfx3d_glVertex3_cord(unsigned int one, unsigned int two, unsigned int v)
 	SetVertex ();
 }
 
-void gfx3d_glVertex_rel(unsigned long v)
+void gfx3d_glVertex_rel(u32 v)
 {
 	coord[0]		+= float10RelTable[v&1023];
 	coord[1]		+= float10RelTable[(v>>10)&1023];
@@ -784,34 +805,15 @@ int gfx3d_GetNumVertex()
 	return 0;
 }
 
-static void gfx3d_glPolygonAttrib_cache()
+
+
+void gfx3d_glPolygonAttrib (u32 val)
 {
-	// Light enable/disable
-	lightMask = (polyAttr&0xF);
-
-	// texture environment
-	envMode = (polyAttr&0x30)>>4;
-
-	//// overwrite depth on alpha pass
-	//alphaDepthWrite = BIT11(polyAttr);
-
-	//// depth test function
-	//depthFuncMode = depthFunc[BIT14(polyAttr)];
-
-	//// back face culling
-	cullingMask = (polyAttr>>6)&3;
-
-	// Alpha value, actually not well handled, 0 should be wireframe
-	colorRGB[3] = colorAlpha = ((polyAttr>>16)&0x1F);
-
-	//// polyID
-	//polyID = (polyAttr>>24)&0x1F;
-}
-
-void gfx3d_glPolygonAttrib (unsigned long val)
-{
-	polyAttr = val;
-	gfx3d_glPolygonAttrib_cache();
+	if(inBegin) {
+		PROGINFO("Set polyattr in the middle of a begin/end pair.\n  (This won't be activated until the next begin)\n");
+		//TODO - we need some some similar checking for teximageparam etc.
+	}
+	polyAttrPending = val;
 }
 
 /*
@@ -823,7 +825,7 @@ void gfx3d_glPolygonAttrib (unsigned long val)
 	21-25 Ambient Reflection Green
 	26-30 Ambient Reflection Blue
 */
-void gfx3d_glMaterial0(unsigned long val)
+void gfx3d_glMaterial0(u32 val)
 {
 	dsDiffuse = val&0xFFFF;
 	dsAmbient = val>>16;
@@ -836,13 +838,13 @@ void gfx3d_glMaterial0(unsigned long val)
 	}
 }
 
-void gfx3d_glMaterial1(unsigned long val)
+void gfx3d_glMaterial1(u32 val)
 {
 	dsSpecular = val&0xFFFF;
 	dsEmission = val>>16;
 }
 
-BOOL gfx3d_glShininess (unsigned long val)
+BOOL gfx3d_glShininess (u32 val)
 {
 	shininessTable[shininessInd++] = ((val & 0xFF) / 256.0f);
 	shininessTable[shininessInd++] = (((val >> 8) & 0xFF) / 256.0f);
@@ -869,18 +871,18 @@ static void gfx3d_glTexImage_cache()
 {
 	texCoordinateTransform = (textureFormat>>30);
 }
-void gfx3d_glTexImage(unsigned long val)
+void gfx3d_glTexImage(u32 val)
 {
 	textureFormat = val;
 	gfx3d_glTexImage_cache();
 }
 
-void gfx3d_glTexPalette(unsigned long val)
+void gfx3d_glTexPalette(u32 val)
 {
 	texturePalette = val;
 }
 
-void gfx3d_glTexCoord(unsigned long val)
+void gfx3d_glTexCoord(u32 val)
 {
 	_t = (s16)(val>>16);
 	_s = (s16)(val&0xFFFF);
@@ -904,25 +906,23 @@ void gfx3d_glTexCoord(unsigned long val)
 
 #define vec3dot(a, b)		(((a[0]) * (b[0])) + ((a[1]) * (b[1])) + ((a[2]) * (b[2])))
 
-void gfx3d_glNormal(unsigned long v)
+void gfx3d_glNormal(u32 v)
 {
 	int i,c;
 	ALIGN(16) float normal[3] = { normalTable[v&1023],
 						normalTable[(v>>10)&1023],
 						normalTable[(v>>20)&1023]};
 
-	//use the current normal transform matrix
-	MatrixMultVec3x3 (mtxCurrent[2], normal);
-
 	if (texCoordinateTransform == 2)
 	{
-		float normal_fixed[3] = {normal[0]/8,normal[1]/8,normal[2]/8};
-		last_s =(	(normal_fixed[0] *mtxCurrent[3][0] + normal_fixed[1] *mtxCurrent[3][4] +
-					 normal_fixed[2] *mtxCurrent[3][8]) + _s);
-		last_t =(	(normal_fixed[0] *mtxCurrent[3][1] + normal_fixed[1] *mtxCurrent[3][5] +
-					 normal_fixed[2] *mtxCurrent[3][9]) + _t);
+		last_s =(	(normal[0] *mtxCurrent[3][0] + normal[1] *mtxCurrent[3][4] +
+					 normal[2] *mtxCurrent[3][8]) + (_s*16.0f)) / 16.0f;
+		last_t =(	(normal[0] *mtxCurrent[3][1] + normal[1] *mtxCurrent[3][5] +
+					 normal[2] *mtxCurrent[3][9]) + (_t*16.0f)) / 16.0f;
 	}
 
+	//use the current normal transform matrix
+	MatrixMultVec3x3 (mtxCurrent[2], normal);
 
 	//apply lighting model
 	{
@@ -1027,7 +1027,7 @@ static void gfx3d_glLightDirection_cache(int index)
 	20-29 Directional Vector's Z component (1bit sign + 9bit fractional part)
 	30-31 Light Number                     (0..3)
 */
-void gfx3d_glLightDirection (unsigned long v)
+void gfx3d_glLightDirection (u32 v)
 {
 	int index = v>>30;
 
@@ -1035,34 +1035,114 @@ void gfx3d_glLightDirection (unsigned long v)
 	gfx3d_glLightDirection_cache(index);
 }
 
-void gfx3d_glLightColor (unsigned long v)
+void gfx3d_glLightColor (u32 v)
 {
 	int index = v>>30;
 	lightColor[index] = v;
 }
 
-void gfx3d_glAlphaFunc(unsigned long v)
+void gfx3d_glAlphaFunc(u32 v)
 {
-	gfx3d.alphaTestRef = (v&31)/31.f;
+	gfx3d.alphaTestRef = v&31;
 }
 
-BOOL gfx3d_glBoxTest(unsigned long v)
+BOOL gfx3d_glBoxTest(u32 v)
 {
 	u32 gxstat = T1ReadLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600);
 	gxstat |= 0x00000001;		// busy
 	T1WriteLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600, gxstat);
 
 	BTind++;
-	if (BTind < 3) return FALSE;
+	if (BTind < 3) 
+	{
+	/*	if(BTind == 1)
+		{
+			BTcoords[0] = float16table[v & 0xFFFF];
+			BTcoords[1] = float16table[v >> 16];
+		}
+		else if(BTind == 2)
+		{
+			BTcoords[2] = float16table[v & 0xFFFF];
+			BTcoords[3] = float16table[v >> 16];
+		}*/
+
+		return FALSE;
+	}
 	BTind = 0;
 
+/*	BTcoords[4] = float16table[v & 0xFFFF];
+	BTcoords[5] = float16table[v >> 16];
+
+	ALIGN(16) float boxCoords[6][4][4] = {
+		// near
+		{{BTcoords[0], BTcoords[1], BTcoords[2], 1.0f}, 
+			{BTcoords[0]+BTcoords[3], BTcoords[1], BTcoords[2], 1.0f}, 
+			{BTcoords[0]+BTcoords[3], BTcoords[1]+BTcoords[4], BTcoords[2], 1.0f}, 
+			{BTcoords[0], BTcoords[1]+BTcoords[4], BTcoords[2], 1.0f}},
+		// far
+		{{BTcoords[0], BTcoords[1], BTcoords[2]+BTcoords[5], 1.0f}, 
+			{BTcoords[0]+BTcoords[3], BTcoords[1], BTcoords[2]+BTcoords[5], 1.0f}, 
+			{BTcoords[0]+BTcoords[3], BTcoords[1]+BTcoords[4], BTcoords[2]+BTcoords[5], 1.0f}, 
+			{BTcoords[0], BTcoords[1]+BTcoords[4], BTcoords[2]+BTcoords[5], 1.0f}},
+		// left
+		{{BTcoords[0], BTcoords[1], BTcoords[2]+BTcoords[5], 1.0f}, 
+			{BTcoords[0], BTcoords[1], BTcoords[2], 1.0f}, 
+			{BTcoords[0], BTcoords[1]+BTcoords[4], BTcoords[2], 1.0f}, 
+			{BTcoords[0], BTcoords[1]+BTcoords[4], BTcoords[2]+BTcoords[5], 1.0f}},
+		// right
+		{{BTcoords[0]+BTcoords[3], BTcoords[1], BTcoords[2], 1.0f}, 
+			{BTcoords[0]+BTcoords[3], BTcoords[1], BTcoords[2]+BTcoords[5], 1.0f}, 
+			{BTcoords[0]+BTcoords[3], BTcoords[1]+BTcoords[4], BTcoords[2]+BTcoords[5], 1.0f}, 
+			{BTcoords[0]+BTcoords[3], BTcoords[1]+BTcoords[4], BTcoords[2], 1.0f}},
+		// top
+		{{BTcoords[0], BTcoords[1], BTcoords[2]+BTcoords[5], 1.0f}, 
+			{BTcoords[0]+BTcoords[3], BTcoords[1], BTcoords[2]+BTcoords[5], 1.0f}, 
+			{BTcoords[0]+BTcoords[3], BTcoords[1], BTcoords[2], 1.0f}, 
+			{BTcoords[0], BTcoords[1], BTcoords[2], 1.0f}},
+		// bottom
+		{{BTcoords[0], BTcoords[1]+BTcoords[4], BTcoords[2]+BTcoords[5], 1.0f}, 
+			{BTcoords[0]+BTcoords[3], BTcoords[1]+BTcoords[4], BTcoords[2]+BTcoords[5], 1.0f}, 
+			{BTcoords[0]+BTcoords[3], BTcoords[1]+BTcoords[4], BTcoords[2], 1.0f}, 
+			{BTcoords[0], BTcoords[1]+BTcoords[4], BTcoords[2], 1.0f}}
+	};
+
+	for(int face = 0; face < 6; face++)
+	{
+		for(int vtx = 0; vtx < 4; vtx++)
+		{
+			MatrixMultVec4x4(mtxCurrent[1], boxCoords[face][vtx]);
+			MatrixMultVec4x4(mtxCurrent[0], boxCoords[face][vtx]);
+
+			boxCoords[face][vtx][0] = ((boxCoords[face][vtx][0] + boxCoords[face][vtx][3]) / (2 * boxCoords[face][vtx][3]));
+			boxCoords[face][vtx][1] = ((boxCoords[face][vtx][1] + boxCoords[face][vtx][3]) / (2 * boxCoords[face][vtx][3]));
+			boxCoords[face][vtx][2] = ((boxCoords[face][vtx][2] + boxCoords[face][vtx][3]) / (2 * boxCoords[face][vtx][3]));
+
+		//	if(face==0)INFO("box test: testing face %i, vtx %i: %f %f %f %f\n", face, vtx, 
+		//		boxCoords[face][vtx][0], boxCoords[face][vtx][1], boxCoords[face][vtx][2], boxCoords[face][vtx][3]);
+
+			if ((boxCoords[face][vtx][0] >= 0.0f) && (boxCoords[face][vtx][0] <= 1.0f) &&
+				(boxCoords[face][vtx][1] >= 0.0f) && (boxCoords[face][vtx][1] <= 1.0f) &&
+				(boxCoords[face][vtx][2] >= 0.0f) && (boxCoords[face][vtx][2] <= 1.0f))
+			{
+			//	goto faceInside;
+			}
+			else
+				goto noFaceInside;
+		}
+	}
+
+	goto faceInside;
+
+faceInside:
+	gxstat |= 0x2;
+
+noFaceInside:*/
 	gxstat &= 0xFFFFFFFE;
 	T1WriteLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600, gxstat);
-	//INFO("BoxTest=%i\n",v);
 	return TRUE;
 }
 
-BOOL gfx3d_glPosTest(unsigned long v)
+BOOL gfx3d_glPosTest(u32 v)
 {
 	u32 gxstat = T1ReadLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600);
 	gxstat |= 0x00000001;		// busy
@@ -1090,7 +1170,7 @@ BOOL gfx3d_glPosTest(unsigned long v)
 	return TRUE;
 }
 
-void gfx3d_glVecTest(unsigned long v)
+void gfx3d_glVecTest(u32 v)
 {
 	u32 gxstat = T1ReadLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600);
 	gxstat &= 0xFFFFFFFE;
@@ -1111,7 +1191,6 @@ unsigned short gfx3d_glGetVecRes(unsigned int index)
 }
 
 #ifdef USE_GEOMETRY_FIFO_EMULATION
-extern void NDS_Pause();
 
 void gfx3d_execute(u8 cmd, u32 param)
 {
@@ -1249,13 +1328,16 @@ static void gfx3d_FlushFIFO()
 	GFX_FIFOclear();
 }
 
-void gfx3d_glFlush(unsigned long v)
+void gfx3d_glFlush(u32 v)
 {
+	gfx3d.frameCtr++;
+
 	gfx3d_FlushFIFO();
 
+	//assert(!flushPending);
 	flushPending = TRUE;
-	gfx3d.wbuffer = (v&1)!=0;
-	gfx3d.sortmode = ((v>>1)&1)!=0;
+	gfx3d.sortmode = BIT0(v);
+	gfx3d.wbuffer = BIT1(v);
 
 	// reset
 	clInd = 0;
@@ -1264,10 +1346,9 @@ void gfx3d_glFlush(unsigned long v)
 	//the renderer wil lget the lists we just built
 	gfx3d.polylist = polylist;
 	gfx3d.vertlist = vertlist;
-//	gfx3d.projlist = projlist;
 
 	//we need to sort the poly list with alpha polys last
-	//first, look for alpha polys
+	//first, look for opaque polys
 	int polycount = polylist->count;
 	int ctr=0;
 	for(int i=0;i<polycount;i++) {
@@ -1300,8 +1381,11 @@ void gfx3d_VBlankSignal()
 	drawPending = TRUE;
 }
 
-void gfx3d_VBlankEndSignal()
+void gfx3d_VBlankEndSignal(bool skipFrame)
 {
+	//if we are skipping 3d frames then the 3d rendering will get held up here.
+	//but, as soon as we quit skipping frames, the held-up 3d frame will render
+	if(skipFrame) return;
 	if(!drawPending) return;
 
 	drawPending = FALSE;
@@ -2025,7 +2109,7 @@ static void gfx3d_Control_cache()
 	}
 }
 
-void gfx3d_Control(unsigned long v)
+void gfx3d_Control(u32 v)
 {
 	control = v;
 	gfx3d_Control_cache();
@@ -2063,6 +2147,8 @@ void gfx3d_glGetLightColor(unsigned int index, unsigned int* dest)
 SFORMAT SF_GFX3D[]={
 	{ "GCTL", 4, 1, &control},
 	{ "GPAT", 4, 1, &polyAttr},
+	{ "GPAP", 4, 1, &polyAttrPending},
+	{ "GINB", 4, 1, &inBegin},
 	{ "GTFM", 4, 1, &textureFormat},
 	{ "GTPA", 4, 1, &texturePalette},
 	{ "GMOD", 4, 1, &mode},
@@ -2111,6 +2197,9 @@ SFORMAT SF_GFX3D[]={
 	{ "GMSP", 2, 1, &dsSpecular},
 	{ "GMEM", 2, 1, &dsEmission},
 	{ "GTST", 4, 1, &triStripToggle},
+	{ "GTVC", 4, 1, &tempVertInfo.count},
+	{ "GTVM", 4, 4, &tempVertInfo.map[0]},
+	{ "GTVF", 4, 1, &tempVertInfo.first},
 	{ "GLTW", 4, 1, &listTwiddle},
 	{ "GFLP", 4, 1, &flushPending},
 	{ "GDRP", 4, 1, &drawPending},
@@ -2122,12 +2211,9 @@ SFORMAT SF_GFX3D[]={
 	{ "GSSH", 4, 1, &gfx3d.shading},
 	{ "GSWB", 4, 1, &gfx3d.wbuffer},
 	{ "GSSM", 4, 1, &gfx3d.sortmode},
-	{ "GSAR", 4, 1, &gfx3d.alphaTestRef},
-	{ "GSVX", 4, 1, &gfx3d.viewport.x},
-	{ "GSVY", 4, 1, &gfx3d.viewport.y},
-	{ "GSVW", 4, 1, &gfx3d.viewport.width},
-	{ "GSVH", 4, 1, &gfx3d.viewport.height},
-	{ "GSCC", 4, 4, gfx3d.clearColor},
+	{ "GSAR", 1, 1, &gfx3d.alphaTestRef},
+	{ "GSVP", 4, 1, &viewport},
+	{ "GSCC", 4, 1, &gfx3d.clearColor},
 	{ "GSCD", 4, 1, &gfx3d.clearDepth},
 	{ "GSFC", 4, 4, gfx3d.fogColor},
 	{ "GSFO", 4, 1, &gfx3d.fogOffset},
@@ -2157,16 +2243,12 @@ bool gfx3d_loadstate(std::istream* is)
 	//jiggle the lists. and also wipe them. this is clearly not the best thing to be doing.
 	polylist = &polylists[listTwiddle];
 	vertlist = &vertlists[listTwiddle];
-//	projlist = &projlists[listTwiddle];
 	polylist->count = 0;
 	vertlist->count = 0;
-//	projlist->count = 0;
 	gfx3d.polylist = &polylists[listTwiddle^1];
 	gfx3d.vertlist = &vertlists[listTwiddle^1];
-//	gfx3d.projlist = &projlists[listTwiddle^1];
 	gfx3d.polylist->count=0;
 	gfx3d.vertlist->count=0;
-//	gfx3d.projlist->count = 0;
 
 	return true;
 }
