@@ -19,12 +19,13 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+ 
 #include "callbacks.h"
 #include "callbacks_IO.h"
 #include "dTools/callbacks_dtools.h"
 #include "globals.h"
 #include "keyval_names.h"
+#include "rasterize.h"
 
 #ifdef GDB_STUB
 #include "../gdbstub.h"
@@ -39,13 +40,31 @@
 /*
  * The frame limiter semaphore
  */
-SDL_sem *glade_fps_limiter_semaphore;
+//SDL_sem *glade_fps_limiter_semaphore;
 int glade_fps_limiter_disabled = 0;
 
 GtkWidget * pWindow;
 GtkWidget * pDrawingArea, * pDrawingArea2;
 GladeXML  * xml, * xml_tools;
 
+const u16 gtk_kb_cfg[NB_KEYS] =
+  {
+    GDK_x,         // A
+    GDK_z,         // B
+    GDK_Shift_R,   // select
+    GDK_Return,    // start
+    GDK_Right,     // Right
+    GDK_Left,      // Left
+    GDK_Up,        // Up
+    GDK_Down,      // Down       
+    GDK_w,         // R
+    GDK_q,         // L
+    GDK_s,         // X
+    GDK_a,         // Y
+    GDK_p,         // DEBUG
+    GDK_o          // BOOST
+  };
+
 SoundInterface_struct *SNDCoreList[] = {
 &SNDDummy,
 &SNDFile,
@@ -54,22 +73,24 @@ NULL
 };
 
 GPU3DInterface *core3DList[] = {
-&gpu3DNull
+&gpu3DNull,
+&gpu3DRasterize
 #ifdef GTKGLEXT_AVAILABLE
   ,
   &gpu3Dgl
 #endif
 };
 
-
 /*
  *
  * Command line handling
  *
  */
 struct configured_features {
+  int load_slot;
   int software_colour_convert;
-  int disable_3d;
+  int opengl_2d;
+  int engine_3d;
   int disable_limiter;
 
   u16 arm9_gdb_port;
@@ -82,12 +103,14 @@ struct configured_features {
 
 static void
 init_configured_features( struct configured_features *config) {
+  config->load_slot = 0;
   config->arm9_gdb_port = 0;
   config->arm7_gdb_port = 0;
 
   config->software_colour_convert = 0;
 
-  config->disable_3d = 0;
+  config->opengl_2d = 0;
+  config->engine_3d = 1;
 
   config->disable_limiter = 0;
 
@@ -108,13 +131,21 @@ fill_configured_features( struct configured_features *config,
     if ( strcmp( argv[i], "--help") == 0) {
       g_print( _("USAGE: %s [OPTIONS] [nds-file]\n"), argv[0]);
       g_print( _("OPTIONS:\n"));
+      g_print( _("\
+   --load-slot=NUM     Load game saved under NUM position.\n\n"));
 #ifdef GTKGLEXT_AVAILABLE
       g_print( _("\
    --soft-convert      Use software colour conversion during OpenGL\n\
                        screen rendering. May produce better or worse\n\
-                       frame rates depending on hardware.\n\
-   \n\
-   --disable-3d        Disables the 3D emulation\n\n"));
+                       frame rates depending on hardware.\n\n"));
+#endif
+      g_print( _("\
+   --3d-engine=ENGINE  Selects 3D rendering engine\n\
+                         0 = disabled\n\
+                         1 = internal desmume rasterizer (default)\n"));
+#ifdef GTKGLEXT_AVAILABLE
+      g_print( _("\
+                         2 = gtkglext off-screen 3d opengl\n\n"));
 #endif
       g_print( _("\
    --disable-limiter   Disables the 60 fps limiter\n\
@@ -136,14 +167,42 @@ fill_configured_features( struct configured_features *config,
       //g_print("   --sticky            Enable sticky keys and stylus\n");
       good_args = 0;
     }
+    else if ( strncmp( argv[i], "--load-slot=", 12) == 0) {
+      char *end_char;
+      int slot = strtoul( &argv[i][12], &end_char, 10);
+
+      if ( slot >= 0 && slot <= 10) {
+        config->load_slot = slot;
+      }
+      else {
+        g_printerr( _("I only know how to load from slots 1-10.\n"));
+        good_args = 0;
+      }
+    }
 #ifdef GTKGLEXT_AVAILABLE
+    else if ( strcmp( argv[i], "--opengl-2d") == 0) {
+        // FIXME: to be implemented
+      config->opengl_2d = 1;
+    }
     else if ( strcmp( argv[i], "--soft-convert") == 0) {
       config->software_colour_convert = 1;
     }
-    else if ( strcmp( argv[i], "--disable-3d") == 0) {
-      config->disable_3d = 1;
-    }
+#define MAX3DEMU 2
+#else
+#define MAX3DEMU 1
 #endif
+    else if ( strncmp( argv[i], "--3d-engine=", 12) == 0) {
+      char *end_char;
+      int engine = strtoul( &argv[i][12], &end_char, 10);
+
+      if ( engine >= 0 && engine <= MAX3DEMU) {
+        config->engine_3d = engine;
+      }
+      else {
+        g_printerr( _("Supported 3d engines: 0, 1, and on some machines 2; use --help option for details\n"));
+        good_args = 0;
+      }
+    }
     else if ( strncmp( argv[i], "--fwlang=", 9) == 0) {
       char *end_char;
       int lang = strtoul( &argv[i][9], &end_char, 10);
@@ -263,7 +322,7 @@ static int Read_ConfigFile()
 	GKeyFile * keyfile = g_key_file_new();
 	GError * error = NULL;
 	
-	load_default_config();
+	load_default_config(gtk_kb_cfg);
 	
 	g_key_file_load_from_file(keyfile, CONFIG_FILE, G_KEY_FILE_NONE, 0);
 
@@ -352,23 +411,23 @@ joinThread_gdb( void *thread_handle) {
  * 
  * @return The interval to the next call (required by SDL)
  */
-static Uint32
+/*static Uint32
 glade_fps_limiter_fn( Uint32 interval, void *param) {
   SDL_sem *sdl_semaphore = (SDL_sem *)param;
 
-  /* signal the semaphore if it is getting low */
+  /* signal the semaphore if it is getting low *-/
   if ( SDL_SemValue( sdl_semaphore) < 4) {
     SDL_SemPost( sdl_semaphore);
   }
 
   return interval;
-}
+}*/
 
 /* ***** ***** MAIN ***** ***** */
 
 static int
 common_gtk_glade_main( struct configured_features *my_config) {
-	SDL_TimerID limiter_timer;
+	/*SDL_TimerID limiter_timer;*/
 #ifdef GDB_STUB
         gdbstub_handle_t arm9_gdb_stub;
         gdbstub_handle_t arm7_gdb_stub;
@@ -475,6 +534,7 @@ common_gtk_glade_main( struct configured_features *my_config) {
 	/* check command line file */
 	if( my_config->nds_file) {
 		if(desmume_open( my_config->nds_file) >= 0)	{
+            loadstate_slot( my_config->load_slot);
 			desmume_resume();
 			enable_rom_features();
 		} else {
@@ -492,31 +552,20 @@ common_gtk_glade_main( struct configured_features *my_config) {
         gtk_widget_show(pDrawingArea2);
 
         {
-          int use_null_3d = my_config->disable_3d;
+          int engine = my_config->engine_3d;
 
 #ifdef GTKGLEXT_AVAILABLE
-          if ( !use_null_3d) {
+          if ( my_config->engine_3d==2 )
             /* setup the gdk 3D emulation */
-            if ( init_opengl_gdk_3Demu(GDK_DRAWABLE(pWindow->window))) {
-              NDS_3D_SetDriver(1);
-
-              if (!gpu3D->NDS_3D_Init()) {
-                fprintf( stderr, _("Failed to initialise openGL 3D emulation; "
-                         "removing 3D support\n"));
-                use_null_3d = 1;
+              if(!init_opengl_gdk_3Demu(GDK_DRAWABLE(pWindow->window))){
+                  fprintf( stderr, _("Failed to initialise openGL 3D emulation; "
+                                     "removing 3D support\n"));
+                  engine = 0;
               }
-            }
-            else {
-              fprintf( stderr, _("Failed to setup openGL 3D emulation; "
-                       "removing 3D support\n"));
-              use_null_3d = 1;
-            }
-          }
 #endif
-          if ( use_null_3d) {
-            NDS_3D_SetDriver ( 0);
-            gpu3D->NDS_3D_Init();
-          }
+          NDS_3D_ChangeCore(engine);
+          if(my_config->engine_3d != 0 && gpu3D == GPU3D_NULL)
+              fprintf(stderr, _("Failed to setup 3D engine; removing 3D support\n"));
         }
 
 //	on_menu_tileview_activate(NULL,NULL);
@@ -524,12 +573,12 @@ common_gtk_glade_main( struct configured_features *my_config) {
         /* setup the frame limiter and indicate if it is disabled */
         glade_fps_limiter_disabled = my_config->disable_limiter;
 
-        if ( !glade_fps_limiter_disabled) {
-          /* create the semaphore used for fps limiting */
+    /*    if ( !glade_fps_limiter_disabled) {
+          /* create the semaphore used for fps limiting *-/
           glade_fps_limiter_semaphore = SDL_CreateSemaphore( 1);
 
           /* start a SDL timer for every FPS_LIMITER_FRAME_PERIOD
-           * frames to keep us at 60 fps */
+           * frames to keep us at 60 fps *-/
           limiter_timer = SDL_AddTimer( 16 * FPS_LIMITER_FRAME_PERIOD,
                                         glade_fps_limiter_fn,
                                         glade_fps_limiter_semaphore);
@@ -539,17 +588,17 @@ common_gtk_glade_main( struct configured_features *my_config) {
             SDL_DestroySemaphore( glade_fps_limiter_semaphore);
             glade_fps_limiter_disabled = 1;
           }
-        }
+        }*/
 
 	/* start event loop */
 	gtk_main();
 	desmume_free();
 
-        if ( !glade_fps_limiter_disabled) {
-          /* tidy up the FPS limiter timer and semaphore */
+      /*  if ( !glade_fps_limiter_disabled) {
+          /* tidy up the FPS limiter timer and semaphore *-/
           SDL_RemoveTimer( limiter_timer);
           SDL_DestroySemaphore( glade_fps_limiter_semaphore);
-        }
+        }*/
 
         /* Unload joystick */
         uninit_joy();
@@ -603,4 +652,3 @@ int WinMain ( HINSTANCE hThisInstance, HINSTANCE hPrevInstance,
   main( argc, argv);
 }
 #endif
-
