@@ -64,14 +64,15 @@ u8 cmdBitsSize[8] = {8, 8, 56, 24, 0, 24, 8, 8};
 
 static void rtcRecv()
 {
-	//INFO("RTC Read command 0x%02X\n", rtc.cmd);
+	//INFO("RTC Read command 0x%02X\n", (rtc.cmd >> 1));
 	memset(rtc.data, 0, sizeof(rtc.data));
-	switch (rtc.cmd)
+	switch (rtc.cmd >> 1)
 	{
 		case 0:				// status register 1
 			//INFO("RTC: read regstatus1 (0x%02X)\n", rtc.regStatus1);
+			rtc.regStatus1 &= 0x0F;
 			rtc.data[0] = rtc.regStatus1;
-			rtc.regStatus1 &= 0x7F;
+			//rtc.regStatus1 &= 0x7F;
 			break;
 		case 1:				// status register 2
 			//INFO("RTC: read regstatus2 (0x%02X)\n", rtc.regStatus1);
@@ -89,12 +90,8 @@ static void rtcRecv()
 				rtc.data[1] = toBCD(tm_local->tm_mon);
 				rtc.data[2] = toBCD(tm_local->tm_mday);
 				rtc.data[3] =  (tm_local->tm_wday + 6) & 7;
-				if (!(rtc.regStatus1 & 0x02))
-				{
-					rtc.data[4] = (tm_local->tm_hour < 12)?0x00:0x40;
-					tm_local->tm_hour %= 12;
-				}
-				rtc.data[4] |= toBCD(tm_local->tm_hour);
+				if (!(rtc.regStatus1 & 0x02)) tm_local->tm_hour %= 12;
+				rtc.data[4] = ((tm_local->tm_hour < 12) ? 0x00 : 0x40) | toBCD(tm_local->tm_hour);
 				rtc.data[5] =  toBCD(tm_local->tm_min);
 				rtc.data[6] =  toBCD(tm_local->tm_sec);
 				break;
@@ -105,12 +102,8 @@ static void rtcRecv()
 				time_t	tm;
 				time(&tm);
 				struct tm *tm_local= localtime(&tm);
-				if (!(rtc.regStatus1 & 0x02))
-				{
-					rtc.data[4] = (tm_local->tm_hour < 12)?0x00:0x40;
-					tm_local->tm_hour %= 12;
-				}
-				rtc.data[0] |= toBCD(tm_local->tm_hour);
+				if (!(rtc.regStatus1 & 0x02)) tm_local->tm_hour %= 12;
+				rtc.data[0] = ((tm_local->tm_hour < 12) ? 0x00 : 0x40) | toBCD(tm_local->tm_hour);
 				rtc.data[1] =  toBCD(tm_local->tm_min);
 				rtc.data[2] =  toBCD(tm_local->tm_sec);
 				break;
@@ -138,23 +131,24 @@ static void rtcRecv()
 
 static void rtcSend()
 {
-	//INFO("RTC write 0x%02X\n", rtc.cmd);
-	switch (rtc.cmd)
+	//INFO("RTC write 0x%02X\n", (rtc.cmd >> 1));
+	switch (rtc.cmd >> 1)
 	{
 		case 0:				// status register 1
 			//INFO("RTC: write regstatus1 0x%02X\n", rtc.data[0]);
-			rtc.regStatus1 &= 0xF1;
-			rtc.regStatus1 |= (rtc.data[0] | 0x0E);
+		//	rtc.regStatus1 &= 0xF1;
+		//	rtc.regStatus1 |= (rtc.data[0] | 0x0E);
+			rtc.regStatus1 = rtc.data[0];
 			break;
 		case 1:				// status register 2
 			//INFO("RTC: write regstatus2 0x%02X\n", rtc.data[0]);
 			rtc.regStatus2 = rtc.data[0];
 			break;
 		case 2:				// date & time
-			//INFO("RTC: write date & time\n");
+			//INFO("RTC: write date & time : %02X %02X %02X %02X %02X %02X %02X\n", rtc.data[0], rtc.data[1], rtc.data[2], rtc.data[3], rtc.data[4], rtc.data[5], rtc.data[6]);
 		break;
 		case 3:				// time
-			//INFO("RTC: write time\n");
+			//INFO("RTC: write time : %02X %02X %02X\n", rtc.data[0], rtc.data[1], rtc.data[2]);
 		break;
 		case 4:				// freq/alarm 1
 			/*if (cmdBitsSize[0x04] == 8)
@@ -182,7 +176,7 @@ void rtcInit()
 	rtc.regStatus1 |= 0x02;
 }
 
-u8 rtcRead() 
+u16 rtcRead() 
 { 
 	//INFO("MMU Read RTC 0x%02X (%03i)\n", rtc._REG, rtc.bitsCount);
 	return (rtc._REG); 
@@ -219,47 +213,44 @@ void rtcWrite(u16 val)
 
 			rtc.cmd |= (rtc._SIO << rtc.bitsCount );
 			rtc.bitsCount ++;
-			if (rtc.bitsCount == 7)
+			if (rtc.bitsCount == 8)
 			{
-				if ( (rtc.cmd & 0x06) != 0x06 )
-				{
-					rtc.cmdStat = 0;
-					//INFO("RTC uknown command 0x02%X\n", rtc.cmd);
-					break;
-				}
-				rtc.cmdStat = 2;
-				rtc.cmd >>= 4;
-				rtc.cmd &= 0x07;
 				//INFO("RTC command 0x%02X\n", rtc.cmd);
-			}
 
-		break;
-		case 2:
-			if (!rtc._CS) 
-			{
-				rtc.cmdStat = 0;
-				break;
-			}
-			if((rtc._prevSCK) && (!rtc._SCK))
-			{
-				rtc.bitsCount = 0;
-				if (rtc.cmd == 0x04)
+				// Little-endian command
+				if((rtc.cmd & 0x0F) == 0x06)
 				{
-					if ((rtc.regStatus2 & 0x0F) == 0x04)
-						cmdBitsSize[rtc.cmd] = 24;
-					else
-						cmdBitsSize[rtc.cmd] = 8;
-				}
-				if (rtc._SIO)
-				{
-					rtc.cmdStat = 4;
-					rtcRecv();
+					u8 tmp = rtc.cmd;
+					rtc.cmd = ((tmp & 0x80) >> 7) | ((tmp & 0x40) >> 5) | ((tmp & 0x20) >> 3) | ((tmp & 0x10) >> 1);
 				}
+				// Big-endian command
 				else
 				{
-					rtc.cmdStat = 3;
+					rtc.cmd &= 0x0F;
+				}
+
+				if((rtc._prevSCK) && (!rtc._SCK))
+				{
+					rtc.bitsCount = 0;
+					if ((rtc.cmd >> 1) == 0x04)
+					{
+						if ((rtc.regStatus2 & 0x0F) == 0x04)
+							cmdBitsSize[rtc.cmd >> 1] = 24;
+						else
+							cmdBitsSize[rtc.cmd >> 1] = 8;
+					}
+					if (rtc.cmd & 0x01)
+					{
+						rtc.cmdStat = 4;
+						rtcRecv();
+					}
+					else
+					{
+						rtc.cmdStat = 3;
+					}
 				}
 			}
+
 		break;
 
 		case 3:			// write:
@@ -267,7 +258,7 @@ void rtcWrite(u16 val)
 			{
 				if(rtc._SIO) rtc.data[rtc.bitsCount >> 3] |= (1 << (rtc.bitsCount & 0x07));
 				rtc.bitsCount++;
-				if (rtc.bitsCount == cmdBitsSize[rtc.cmd])
+				if (rtc.bitsCount == cmdBitsSize[rtc.cmd >> 1])
 				{
 					rtcSend();
 					rtc.cmdStat = 0;
@@ -285,7 +276,7 @@ void rtcWrite(u16 val)
 					rtc._REG &= ~0x01;
 
 				rtc.bitsCount++;
-				if (rtc.bitsCount == cmdBitsSize[rtc.cmd])
+				if (rtc.bitsCount == cmdBitsSize[rtc.cmd >> 1])
 					rtc.cmdStat = 0;
 			}
 		break;
