@@ -20,15 +20,18 @@
 #ifndef WIFI_H
 #define WIFI_H
 
+#include <stdio.h>
 #include "types.h"
 
 #ifdef EXPERIMENTAL_WIFI
 
 /* standardize socket interface for linux and windows */
+/* TODO: get rid of the socket thing, as we're going to use libpcap */
 #ifdef WIN32
 	#include <winsock2.h>
 	#define socket_t 	SOCKET
 	#define sockaddr_t  SOCKADDR
+	#include "windriver.h"
 #else
 	#include <unistd.h>
 	#include <stdlib.h>
@@ -44,6 +47,12 @@
 #define BASEPORT        7000    		/* channel 1: 7000 ... channel 13: 7012 */
 										/* FIXME: make it configureable */
 
+#define HAVE_REMOTE
+#define WPCAP
+#define PACKET_SIZE 65535
+#define _INC_STDIO
+
+
 #define		REG_WIFI_ID					0x000
 #define     REG_WIFI_MODE       		0x004
 #define     REG_WIFI_WEP        		0x006
@@ -58,6 +67,7 @@
 #define     REG_WIFI_AID_LOW       		0x028
 #define     REG_WIFI_AID_HIGH      		0x02A
 #define     REG_WIFI_RETRYLIMIT 		0x02C
+#define		REG_WIFI_RXCNT				0x030
 #define		REG_WIFI_WEPCNT				0x032
 #define		REG_WIFI_POWER_US			0x036
 #define     REG_WIFI_POWERSTATE 		0x03C
@@ -86,6 +96,8 @@
 #define     REG_WIFI_TXLOC3             0x0A8
 #define     REG_WIFI_TXOPT              0x0AC
 #define     REG_WIFI_TXCNT              0x0AE
+#define		REG_WIFI_TXREQ_READ			0x0B0
+#define		REG_WIFI_TXBUSY				0x0B6
 #define     REG_WIFI_TXSTAT             0x0B8
 #define     REG_WIFI_RXFILTER           0x0D0
 #define     REG_WIFI_USCOUNTERCNT       0x0E8
@@ -109,6 +121,7 @@
 #define     REG_WIFI_RFIOBSY    		0x180
 #define     REG_WIFI_RFIOCNT    		0x184
 #define		REG_WIFI_IF_SET				0x21C
+#define		REG_WIFI_POWERACK			0x2D0
 
 /* WIFI misc constants */
 #define		WIFI_CHIPID					0x1440		/* emulates "old" wifi chip, new is 0xC340 */
@@ -370,6 +383,10 @@ typedef union
 /* http://www.akkit.org/info/dswifi.htm#WifiIOMap */
 typedef struct 
 {
+	/* power */
+	BOOL powerOn;
+	BOOL powerOnPending;
+
 	/* wifi interrupt handling */
     wifiirq_t   IE ;
     wifiirq_t   IF ;
@@ -383,8 +400,14 @@ typedef struct
 	u16 TXSlot[3] ;
 	u16 TXCnt ;
 	u16 TXOpt ;
+	u16 TXStat;
 	u16 BEACONSlot ;
 	BOOL BEACON_enable ;
+	u8 txCurSlot;
+	u8 txSlotBusy[3];
+	u32 txSlotAddr[3];
+	u32 txSlotLen[3];
+	u32 txSlotRemainingBytes[3];
 
 	/* receiving */
 	u16 RXCnt ;
@@ -424,11 +447,15 @@ typedef struct
 	rfIOData_t  rfIOData ;
     bbIOCnt_t   bbIOCnt ;
 
+	u8 bbDataToWrite;
+
 	/* buffers */
 	u16         circularBuffer[0x1000] ;
 	u16         RXRangeBegin ;
 	u16         RXRangeEnd ;
 	u16         RXHWWriteCursor ;
+	u16			RXHWWriteCursorReg;
+	u16			RXHWWriteCursorLatched;
 	u16         RXReadCursor ;
 	u16         RXUnits ;
 	u16			RXBufCount ;
@@ -439,10 +466,34 @@ typedef struct
 	u16         CircBufWrEnd ;
 	u16         CircBufWrSkip ;
 
+	/* tx packets */
+	u8 *curPacket[3];
+	int curPacketSize[3];
+	int curPacketPos[3];
+	BOOL curPacketSending[3];
+
+	/* i/o mem */
+	u16			ioMem[0x800];
+
 	/* others */
 	u16			randomSeed ;
 
+	/* SoftAP */
+	struct _SoftAP
+	{
+		pcap_t *bridge;
+
+		u64 usecCounter;
+
+		u8 *curPacket;
+		int curPacketSize;
+		int curPacketPos;
+		BOOL curPacketSending;
+
+	} SoftAP;
+
 	/* desmume host communication */
+	bool		netEnabled;
 	socket_t    udpSocket ;
 	u8			channel ;
 
@@ -452,6 +503,8 @@ extern wifimac_t wifiMac ;
 
 void WIFI_Init(wifimac_t *wifi);
 
+//void WIFI_Thread(wifimac_t *wifi);
+
 /* subchip communication IO functions */
 void WIFI_setRF_CNT(wifimac_t *wifi, u16 val) ;
 void WIFI_setRF_DATA(wifimac_t *wifi, u16 val, u8 part) ;
@@ -469,6 +522,24 @@ u16  WIFI_read16(wifimac_t *wifi,u32 address) ;
 /* wifimac timing */
 void WIFI_usTrigger(wifimac_t *wifi) ;
 
+/* SoftAP */
+
+typedef struct _WIFI_FrameHeader
+{
+	u8 FrameControl[2];
+	u8 DurationID[2];
+	u8 Receiver[6];
+	u8 Sender[6];
+	u8 BSSID[6];
+	u8 SeqCtl[2];
+
+} WIFI_FrameHeader;
+
+int WIFI_SoftAP_Init(wifimac_t *wifi);
+void WIFI_SoftAP_Shutdown(wifimac_t *wifi);
+void WIFI_SoftAP_RecvPacketFromDS(wifimac_t *wifi, u8 *packet, int len);
+void WIFI_SoftAP_usTrigger(wifimac_t *wifi);
+
 #endif
 
 /* DS WFC profile data documented here : */
@@ -499,7 +570,7 @@ typedef struct _FW_WFCProfile
 } FW_WFCProfile;
 
 /* wifi data to be stored in firmware, when no firmware image was loaded */
-extern const u8 FW_Mac[6];
+extern u8 FW_Mac[6];
 extern const u8 FW_WIFIInit[32] ;
 extern const u8 FW_BBInit[105] ;
 extern const u8 FW_RFInit[36] ;
